/*
 * cli.c
 *
 *  Created on: 29.05.2010
 *      Author: AndreR
 */

#include "cli.h"

#include "util/console.h"

#include "stm32f4xx_rcc.h"

#include "hal/led.h"
#include "commands.h"
#include "util/console.h"
#include "util/fifo8_synced.h"
#include "util/bootloader.h"
#include "util/flash_fs.h"
#include "util/flash.h"
#include "hal/usart1.h"
#include "util/boot.h"
#include "util/sys_time.h"
#include "util/log.h"
#include "cpu_load.h"
#include "util/nrf24_ex.h"
//#include "bootloader.h"

#include "network.h"
#include "wifi.h"

#include <stdio.h>
#include <string.h>

CLIGlobal cli;

static void processCommand();

// IRQ Level: USART1 IRQ
static void usart1Callback()
{
	// called on new complete packet in usart1 RX fifo
}

// IRQ Level: Task
static void consoleCallback()
{
	// called after ConsolePrint
}

void CLIInit()
{
	static uint8_t rxData[CONSOLE_RX_SIZE];
	static uint8_t txData[CONSOLE_TX_SIZE];

	ConsoleData conData;
	conData.pDataTx = txData;
	conData.txSize = CONSOLE_TX_SIZE;
	conData.pDataRx = rxData;
	conData.rxSize = CONSOLE_RX_SIZE;
	conData.printSize = CONSOLE_PRINT_SIZE;
	conData.txCallback = &consoleCallback;
	conData.pUserData = 0;

	ConsoleInit(conData);

	usart1.callback = &usart1Callback;
}

msg_t CLITask(void* params)
{
	(void)params;

	chRegSetThreadName("CLI");

	int16_t result = 0;
	uint16_t length;

	systime_t waitUntil = chTimeNow()+MS2ST(10);

	LEDMustOff(LED_MAIN_GREEN | LED_MAIN_RED);

//	NetworkPrint("Firmware Date: %s - %s\r\n", __DATE__, __TIME__);

	while(1)
	{
		// check user input to terminal
		while(FifoPacketSyncedNumPackets(&usart1.rxFifo) > 0)
		{
			// get packet
			result = FifoPacketSyncedPop(&usart1.rxFifo, &length, cli.procBuf, CLI_PROC_SIZE-1);
			if(result == ERROR_FIFO_PACKET_SYNCED_NOT_ENOUGH_MEMORY)
			{
				FifoPacketSyncedDrop(&usart1.rxFifo);
				continue;
			}

			if(result)
				continue;

			ConsolePrint("\r\n");

			cli.procBuf[length] = '\0';

			// process onboard
			uint8_t* pData;
			result = FifoLinReserve(&console.rxFifo, length, &pData);
			if(result)
				continue;

			memcpy(pData, cli.procBuf, length);

			FifoLinCommit(&console.rxFifo, length);
		}

		// check data from main console to terminal
		while(console.txFifo.numPackets > 0)
		{
			uint8_t* pData;
			uint32_t len32;
			result = FifoLinGet(&console.txFifo, &pData, &len32);
			if(result)
				continue;

			FifoSyncedPush(&usart1.txFifo, len32, pData);

			FifoLinDelete(&console.txFifo);

			USART1StartTx();
		}

		if(ConsoleIsCmdReady())
		{
			processCommand();

			ConsoleCmdProcessed();
		}

		chThdSleepUntil(waitUntil);
		waitUntil = chTimeNow()+MS2ST(10);
	}

	return 0;
}

static void miscCommands()
{
	if(ConsoleCmpCmd("reset"))
	{
		BootReset();
	}

	if(ConsoleCmpCmd("bootloader"))
	{
		ConsolePrint("Selecting bootloader\r\n");

		BootSetBootloaderSelected(1);
	}

	if(ConsoleCmpCmd("test"))
	{
		ConsolePrint("Number: %d\r\n", 42);
	}

	float f = 21.0f;
	if(ConsoleScanCmd("float %f", &f) == 1)
	{
		ConsolePrint("Float: %f\r\n", f);
	}

	if(ConsoleCmpCmd("float"))
	{
		ConsolePrint("Float: %f\r\n", 42.0f);
	}

	if(ConsoleCmpCmd("led get"))
	{
		ConsolePrint("LED State: 0x%X\r\n", LEDGetState());
	}

	if(ConsoleCmpCmd("cpu"))
	{
		ConsolePrint("CPU Load: %3u.%1u%%\r\n", cpuLoad.load/10, cpuLoad.load%10);
		ConsolePrint("Max: %3u.%1u%%\r\n", cpuLoad.maxLoad/10, cpuLoad.maxLoad%10);
		ConsolePrint("incPerTick: %10u/%u\r\n", cpuLoad.incPerSysTick, cpuLoad.maxIncPerSysTick);
		cpuLoad.maxLoad = 0;
	}

	if(ConsoleCmpCmd("tasks"))
	{
		ConsolePrint("Name            rPrio\tTime\r\n");

		for(Thread* pThread = chRegFirstThread(); pThread != 0; pThread = chRegNextThread(pThread))
		{
			ConsolePrint("%-16s%u\t%u\r\n", pThread->p_name, pThread->p_realprio, pThread->p_time);
		}
	}

	uint8_t reg;
	if(ConsoleScanCmd("nrf read %hhu", &reg) == 1)
	{
//		uint8_t value;
//		NRF24ExReadRegister(&network.nrf24, reg, &value);
//		ConsolePrint("Reg 0x%X: 0x%X\r\n", reg, value);
	}

	if(ConsoleCmpCmd("wifi stats"))
	{
//		ConsolePrint("Avg: %u\r\n", network.avgRxPeriod);

//		ConsolePrint("-- TX --\r\n");
//		ConsolePrint("Bytes:   %10u/%10u\r\n", network.nrf24.stats.txBytesSent, network.nrf24.stats.txBytesLost);
//		ConsolePrint("Packets: %10u/%10u\r\n", network.nrf24.stats.txPacketsSent, network.nrf24.stats.txPacketsLost);
//		ConsolePrint("Retrans: %10u\r\n",network.nrf24.stats.retransmissions);
//		ConsolePrint("-- RX --\r\n");
//		ConsolePrint("Bytes:   %10u/%10u\r\n", network.nrf24.stats.rxBytesRecv, network.nrf24.stats.rxBytesLost);
//		ConsolePrint("Packets: %10u/%10u\r\n", network.nrf24.stats.rxPacketsRecv, network.nrf24.stats.rxPacketsLost);
	}

	if(ConsoleCmpCmd("float16"))
	{
		float x = 1.0f;
		float16_t f16 = floatToHalfFloat(x);

		float y = halfFloatToFloat(f16);

		ConsolePrint("In: %f, F16: 0x%04X, out: %f\r\n", x, f16, y);
	}

	if(ConsoleCmpCmd("config"))
	{
		ConsolePrint("--- Configuration ---\r\n");
		ConsolePrint("IP:      %hu.%hu.%hu.%hu:%hu\r\n", (uint16_t)network.my.ip.u8[0], (uint16_t)network.my.ip.u8[1],
				(uint16_t)network.my.ip.u8[2], (uint16_t)network.my.ip.u8[3], network.my.port);
		ConsolePrint("Freq. channel: %hu (2.%03huGHz)\r\n", (uint16_t)wifi.cfg.channel, ((uint16_t)wifi.cfg.channel)+400);
		ConsolePrint("Max. RT: %hu\r\n", (uint16_t)wifi.cfg.maxRt);

		if(wifi.cfg.speed == NRF24_SPEED_250K)
			ConsolePrint("Speed: 250k\r\n");
		if(wifi.cfg.speed == NRF24_SPEED_1M)
			ConsolePrint("Speed: 1M\r\n");
		if(wifi.cfg.speed == NRF24_SPEED_2M)
			ConsolePrint("Speed: 2M\r\n");

		ConsolePrint("ACK delay: %huus\r\n", wifi.cfg.ackDelayUs);
		ConsolePrint("Vision freq: %huHz\r\n", (uint16_t)wifi.cfg.maxVisionRate);

		ConsolePrint("Max no-link tries: %hu\r\n", wifi.cfg.maxNoLinkTries);
	}

	IPv4Address ip;
	uint16_t ipIn[4];
	if(ConsoleScanCmd("set ip %hu %hu %hu %hu", ipIn, ipIn+1, ipIn+2, ipIn+3) == 4)
	{
		ip.u8[0] = ipIn[0];
		ip.u8[1] = ipIn[1];
		ip.u8[2] = ipIn[2];
		ip.u8[3] = ipIn[3];
		NetworkChangeIP(ip);

		ConsolePrint("IP: %hu.%hu.%hu.%hu:%hu\r\n", (uint16_t)network.my.ip.u8[0], (uint16_t)network.my.ip.u8[1],
				(uint16_t)network.my.ip.u8[2], (uint16_t)network.my.ip.u8[3], network.my.port);
	}

	uint16_t port;
	if(ConsoleScanCmd("set port %hu", &port) == 1)
	{
		NetworkChangePort(port);

		ConsolePrint("New port: %hu\r\n", network.my.port);
	}

	uint16_t ch;
	if(ConsoleScanCmd("set ch %hu", &ch) == 1)
	{
		WifiChangeChannel(ch);

		ConsolePrint("New channel: %hhu\r\n", wifi.cfg.channel);
	}

	if(ConsoleCmpCmd("config write"))
	{
		ConsolePrint("Writing config to FLASH...");

		NetworkSaveConfig();
		WifiSaveConfig();

		ConsolePrint("OK\r\n");
	}

	uint16_t u16;
	if(ConsoleScanCmd("fixed runtime %hu", &u16) == 1)
	{
		wifi.fixedRuntime = u16;

		ConsolePrint("Fixed Wifi runtime: %hu\r\n", u16);
	}

	if(ConsoleCmpCmd("flash hex"))
	{
		for(uint32_t i = 0; i < flashFS.bytesDel+flashFS.bytesUsed+48; i += 8)
		{
			uint8_t* d = (uint8_t*)(flashFS.flashAddr+i);
			ConsolePrint("0x%04X: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",
					i, d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7]);
		}
	}

	if(ConsoleCmpCmd("flash list"))
	{
		FlashFSListFiles();
	}

	if(ConsoleCmpCmd("flash erase"))
	{
		ConsolePrint("Erasing flash\r\n");

		FlashErase(flashFS.flashAddr, flashFS.flashAddr+flashFS.flashSize);
	}

	if(ConsoleCmpCmd("flash compact"))
	{
		FlashFSCompact();

		ConsolePrint("Flash compacted\r\n");
	}

	if(ConsoleCmpCmd("time"))
	{
		ConsolePrint("Time: %uus\r\n", SysTimeUSec());
	}

	if(ConsoleCmpCmd("log"))
	{
		LogPrintAllRecent(4);
	}

#ifdef ENV_BOOT
	if(ConsoleCmpCmd("start bootloader"))
	{
		ConsolePrint("Starting programming on MAIN\r\n");

//		BootloaderStartProgramming(INTERCOM_ADDRESS_MAIN);
	}
#endif
}

static void processCommand()
{
	miscCommands();

	if(ConsoleCmpCmd("version"))
	{
		ConsolePrint("Firmware Date: %s - %s\r\n", __DATE__, __TIME__);
	}
}
