/*
 * wifi.h
 *
 *  Created on: 07.07.2014
 *      Author: AndreR
 */

#ifndef WIFI_H_
#define WIFI_H_

#include "util/nrf24_ex.h"
#include "util/fifo_packet_synced.h"
#include "util/spi_sync.h"
#include "util/flash_fs.h"
#include "constants.h"

#include "ch.h"

typedef void(*BotDataAvailCallback)(uint8_t);

typedef struct _WifiConfig
{
	uint8_t channel;		// frequency
	uint8_t speed;
	uint8_t maxRt;
	uint8_t maxVisionRate;
	uint16_t ackDelayUs;
	uint16_t maxNoLinkTries;

	// the config struct should always be padded to 4 bytes
} __attribute__((aligned(4),packed)) WifiConfig;

typedef struct _WifiRobot
{
	uint8_t txData[WIFI_TX_SIZE];
	uint8_t rxData[WIFI_RX_SIZE];

	NRF24Queue queue;

	uint8_t online;
//	uint32_t noLinkCounter;
	uint32_t lastResponseTime;
	uint32_t lastVisionTime;
	uint32_t connectCounter;
} WifiRobot;

typedef struct _WifiGlobal
{
	NRF24Ex nrf24;

	WifiRobot bots[NRF24_MAX_BOT_ID+1];

	uint32_t runtime;	// in [us]
	uint8_t maxBots;
	uint32_t timeout;	// in [ms]
	uint8_t fixedRuntime;

	BinarySemaphore timerSem;

	uint8_t scan;

	uint32_t visionPauseTime;	// 50kHz, HFTicks, 20us

	uint32_t lastVisionObjTime;

	SPISync spi3;

	WifiConfig cfg;
	FlashFile* pCfgFile;

	BotDataAvailCallback cbBotData;
}WifiGlobal;

extern WifiGlobal wifi;

void WifiInit();
msg_t WifiTask(void* params);
void WifiSaveConfig();
void WifiChangeChannel(uint8_t channel);
void WifiChangeSpeed(uint8_t speed);
int16_t WifiChangeRetransmission(uint8_t maxRt, uint16_t delayUs);
void WifiSetVisionRate(uint16_t rateHz);
void WifiSendPose(uint16_t botId, float x, float y, float orient);
void WifiSendObjectPose(float objX, float objY, float objOrient);
void WifiSetBotDataCallback(BotDataAvailCallback cb);

#endif /* WIFI_H_ */
