/*
 * ctrl_6step.c
 *
 *  Created on: 11.06.2014
 *      Author: AndreR
 */

#include "ctrl_6step.h"

#include "ch.h"

Ctrl6Step stepDribbler;

static void onInit(Motor* pDrive);
static void onExit(Motor* pDrive);
static void onCurrentUpdate(Motor* pDrive);
static void onSpeedUpdate(Motor* pDrive);
static void onHallUpdateCb(Motor* pDrive);

MotorCtrl ctrl6StepDribbler = {&onInit, &onExit, &onCurrentUpdate, &onSpeedUpdate, &onHallUpdateCb, &stepDribbler};

// W3  W2  W1
//  5   5   5
const uint32_t ccerLUT[8] = {0x555, 0x505, 0x055, 0x550, 0x550, 0x055, 0x505, 0x000 };

// W1, W2, W3 modifier
const float hallMod[8][3] = {
	{0, 0, 0},
	{1, 0, 0},
	{0, 1, 0},
	{0, 1, 0},
	{0, 0, 1},
	{1, 0, 0},
	{0, 0, 1},
	{0, 0, 0}
};

const uint8_t phaseLUT[8] = { 0, 5, 1, 3, 2, 6, 4, 0 };

IN_CCM static void onCurrentUpdate(Motor* pDrive)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;
}

IN_CCM static void onSpeedUpdate(Motor* pDrive)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;

	if(p6Step->speedOn)
	{
		p6Step->pidSpeed.setpoint = p6Step->setVelocity;
		PIDUpdate(&p6Step->pidSpeed, pDrive->curVelocity);

		p6Step->setVoltage = p6Step->pidSpeed.output/MOTOR_KN;
		onHallUpdateCb(pDrive);
//
//		p6Step->setCurrent = p6Step->pidSpeed.output;
//		p6Step->pidCurrent.setpoint = p6Step->setCurrent;
	}
	else
	{
//		p6Step->pidCurrent.setpoint = p6Step->setCurrent;
	}
}

IN_CCM static void onHallUpdateCb(Motor* pDrive)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;

	uint16_t pos = pDrive->halPos;
	float voltage = p6Step->setVoltage;

	if(voltage < 0)
	{
		pos ^= 0x7;
		voltage *= -1.0f;
	}

	MotorEnablePhase(pDrive, ccerLUT[pos]);

	const float* pMod = hallMod[pos];

	MotorSetPhaseVoltagesDirect(pDrive, voltage*pMod[0], voltage*pMod[1], voltage*pMod[2]);
}

static void onInit(Motor* pDrive)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;

	PIDParams currentPIDParams;
	currentPIDParams.kp = 0.5f;
	currentPIDParams.ki = 0.1f;
	currentPIDParams.kd = 0.0f;
	currentPIDParams.outputMin = -2.0f;	// [V]
	currentPIDParams.outputMax = 2.0f;	// [V]

	PIDInit(&p6Step->pidCurrent, &currentPIDParams, 0.00005f, 1);

	PIDParams speedPIDParams;
	speedPIDParams.kp = 1.0f;
	speedPIDParams.ki = 0.1f;
	speedPIDParams.kd = 0.0f;
	speedPIDParams.outputMin = -40000;	// [rpm]
	speedPIDParams.outputMax =  40000;	// [rpm]

	PIDInit(&p6Step->pidSpeed, &speedPIDParams, 0.001f, 1);

	chThdSleep(MS2ST(10));

	MotorSetPower(pDrive, 0);

	Ctrl6StepEnableCurrent(p6Step, 1);
	Ctrl6StepEnableSpeedControl(p6Step, 1);
}

static void onExit(Motor* pDrive)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;

	Ctrl6StepEnableCurrent(p6Step, 0);
	Ctrl6StepEnableSpeedControl(p6Step, 0);
}

void Ctrl6StepEnableCurrent(Ctrl6Step* p6Step, uint8_t enable)
{
	PIDEnable(&p6Step->pidCurrent, enable);

	p6Step->currentOn = enable;
}

void Ctrl6StepEnableSpeedControl(Ctrl6Step* p6Step, uint8_t enable)
{
	PIDEnable(&p6Step->pidSpeed, enable);

	p6Step->speedOn = enable;
}

void Ctrl6StepSetFOCPIDParams(Ctrl6Step* p6Step, float p, float i, float d)
{
	PIDSetParams(&p6Step->pidCurrent, p, i, d);
}

void Ctrl6StepSetSpeedPIDParams(Ctrl6Step* p6Step, float p, float i, float d)
{
	PIDSetParams(&p6Step->pidSpeed, p, i, d);
}

void Ctrl6StepSetPhase(Motor* pDrive, uint8_t ph)
{
	Ctrl6Step* p6Step = (Ctrl6Step*)pDrive->pCtrl->pUserData;

	uint8_t phase = phaseLUT[ph];

	MotorEnablePhase(pDrive, ccerLUT[phase]);

	const float* pMod = hallMod[phase];

	MotorSetPhaseVoltagesDirect(pDrive, p6Step->setVoltage*pMod[0], p6Step->setVoltage*pMod[1], p6Step->setVoltage*pMod[2]);

}

