/*
 * usart1.c
 *
 *  Created on: 26.01.2014
 *      Author: AndreR
 */

#include "usart1.h"

#include "stm32f30x_gpio.h"
#include "stm32f30x_rcc.h"
#include "stm32f30x_usart.h"
#include "stm32f30x_dma.h"
#include "stm32f30x_syscfg.h"
#include "stm32f30x_tim.h"
#include "stm32f30x_misc.h"
#include "constants.h"

#include "hal/led.h"

USART usart1;

void USART1_IRQHandler()
{
	USARTIRQ(&usart1);
}
/*
void DMA1_Channel5_IRQHandler()
{
	USARTDMARx(&usart1);
}

// USART2 TX
void DMA1_Channel4_IRQHandler()
{
	USARTDMATx(&usart1);
}
*/
void TIM6_DAC_IRQHandler()
{
	TIM6->SR = 0;
	TIM_Cmd(TIM6, DISABLE);

	USARTTimerIRQ(&usart1);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM6->ARR = microSeconds;

	TIM_Cmd(TIM6, ENABLE);
}

void USART1Init()
{
	// TIM6 setup
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1000;	// = 1ms
	timeBaseInit.TIM_Prescaler = 71; // clock is now 1MHz
	TIM_TimeBaseInit(TIM6, &timeBaseInit);

	TIM6->SR = 0;
	TIM_ARRPreloadConfig(TIM6, DISABLE);
	TIM_ITConfig(TIM6, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM6_DAC_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_USART1;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	// enable peripheral clock
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

	// provide data storage
	static uint8_t usart2TxData[USART1_FIFO_SIZE];
	static uint8_t usart2RxData[USART1_FIFO_SIZE];

	// map data and configure
	USARTData data2;
	data2.pDataRx = usart2RxData;
	data2.rxSize = USART1_FIFO_SIZE;
	data2.pDataTx = usart2TxData;
	data2.txSize = USART1_FIFO_SIZE;
	data2.maxPacketSize = USART1_MAX_PACKET_SIZE;
	data2.pDMA = DMA1;
	data2.rxStream = 5;
	data2.txStream = 4;
	data2.rxIRQn = DMA1_Channel5_IRQn;
	data2.txIRQn = DMA1_Channel4_IRQn;
	data2.usartIRQn = USART1_IRQn;
	data2.IRQL = IRQL_USART1;
	data2.baudrate = 3000000;
	data2.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart1, USART1, data2);

	// GPIO setup
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	gpioInit.GPIO_Speed = GPIO_Speed_Level_3;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOB, &gpioInit);

	// Map alternate function
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_7);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_7);
}
