/*
, * supply.c
 *
 *  Created on: 28.01.2014
 *      Author: AndreR
 */

#include "supply.h"

#include "stm32f30x_rcc.h"
#include "stm32f30x_adc.h"
#include "stm32f30x_gpio.h"
#include "stm32f30x_opamp.h"
#include "stm32f30x_dma.h"
#include "stm32f30x_misc.h"
#include "stm32f30x_tim.h"

#include "util/intercom.h"
#include "constants.h"

#include "intercom_constants.h"
#include "intercom1.h"

#define EVENT_SAMPLING_DONE		0

//#define ADC_BAT_TO_VOLTAGE 0.0061577690972222f
#define ADC_BAT_TO_VOLTAGE 0.0048014322916667f
//#define ADC_SHUNT_TO_CURRENT 0.00732421875f
#define ADC_SHUNT_TO_CURRENT 0.0146484375f

Supply supply;

static void eventSink(void* pUserData, uint16_t event);
static void periodic(void* pUserData);
static void init(void* pUserData);

void DMA1_Channel1_IRQHandler()
{
	DMA1->IFCR = DMA1_IT_GL1;

	register uint16_t sum = 0;

	for(uint8_t i = 0; i < NUM_DATA_SAMPLES; i++)
		sum += supply.adcData[i];

	supply.batResult = sum/NUM_DATA_SAMPLES;

	sum = 0;
	for(uint8_t i = NUM_DATA_SAMPLES; i < 2*NUM_DATA_SAMPLES; i++)
		sum += supply.adcData[i];

	supply.shuntResult = sum/NUM_DATA_SAMPLES;

	supply.bat = supply.batResult*ADC_BAT_TO_VOLTAGE;
}

void DMA2_Channel1_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	DMA2->IFCR = DMA2_IT_GL1;

	uint16_t ir = 0;

	for(uint8_t i = 0; i < NUM_IR_SAMPLES; i++)
		ir += supply.irData[i];

	ir /= NUM_IR_SAMPLES;

	supply.irResult[supply.irCur++] = ir;
	supply.irCur %= 2;

	if(supply.irDoneCb)
		(*supply.irDoneCb)();

	CH_IRQ_EPILOGUE();
}

void SupplyInit(EventManager* pEventManager)
{
	GPIO_InitTypeDef gpioInit;
	OPAMP_InitTypeDef opInit;

#ifdef DEBUG
	supply.pDMACh = DMA1_Channel1;
	supply.pADC = ADC1;
	supply.pDMA = DMA1;
#endif

	supply.bat = 12.0f;
	supply.pEventClient = EventClientInit(pEventManager, &eventSink, &periodic, MS2ST(100), 0);

	// OPAMP1 pins
	gpioInit.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2;
	gpioInit.GPIO_Mode = GPIO_Mode_AN;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &gpioInit);

	opInit.OPAMP_InvertingInput = OPAMP_InvertingInput_Vout;
	opInit.OPAMP_NonInvertingInput = OPAMP_NonInvertingInput_IO4;
	OPAMP_Init(OPAMP_Selection_OPAMP1, &opInit);

	OPAMP_Cmd(OPAMP_Selection_OPAMP1, ENABLE);

	// Configure ADC Channel4 as analog input
	gpioInit.GPIO_Pin = GPIO_Pin_3;
	gpioInit.GPIO_Mode = GPIO_Mode_AN;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &gpioInit);

	// ADC2 for IR
	gpioInit.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_2;
	GPIO_Init(GPIOB, &gpioInit);

	init(0);
}

static void init(void* pUserData)
{
	(void)pUserData;

	ADC_InitTypeDef adcInit;
	ADC_CommonInitTypeDef adcCommonInit;
	DMA_InitTypeDef dmaInit;
	NVIC_InitTypeDef nvicInit;
	TIM_TimeBaseInitTypeDef timeBaseInit;
	TIM_OCInitTypeDef ocInit;
	GPIO_InitTypeDef gpioInit;

	// DMA configuration
	dmaInit.DMA_PeripheralBaseAddr = (uint32_t)&ADC1->DR;
	dmaInit.DMA_MemoryBaseAddr = (uint32_t)supply.adcData;
	dmaInit.DMA_DIR = DMA_DIR_PeripheralSRC;
	dmaInit.DMA_BufferSize = 2*NUM_DATA_SAMPLES;
	dmaInit.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	dmaInit.DMA_MemoryInc = DMA_MemoryInc_Enable;
	dmaInit.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	dmaInit.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	dmaInit.DMA_Mode = DMA_Mode_Circular;
	dmaInit.DMA_Priority = DMA_Priority_Medium;
	dmaInit.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel1, &dmaInit);

	dmaInit.DMA_PeripheralBaseAddr = (uint32_t)&ADC2->DR;
	dmaInit.DMA_MemoryBaseAddr = (uint32_t)supply.irData;
	dmaInit.DMA_BufferSize = NUM_IR_SAMPLES;
	dmaInit.DMA_Mode = DMA_Mode_Circular;
	DMA_Init(DMA2_Channel1, &dmaInit);

	DMA_ITConfig(DMA1_Channel1, DMA_IT_TC, ENABLE);
	DMA_ITConfig(DMA2_Channel1, DMA_IT_TC, ENABLE);

	// Enable ADC1 clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ADC12, ENABLE);

	// ADC Channel configuration
	adcCommonInit.ADC_Mode = ADC_Mode_Independent;
	adcCommonInit.ADC_Clock = ADC_Clock_SynClkModeDiv1;
	adcCommonInit.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
	adcCommonInit.ADC_DMAMode = ADC_DMAMode_Circular;
	adcCommonInit.ADC_TwoSamplingDelay = 0;
	ADC_CommonInit(ADC1, &adcCommonInit);
	ADC_CommonInit(ADC2, &adcCommonInit);

	// Calibration procedure
	ADC_VoltageRegulatorCmd(ADC1, ENABLE);
	ADC_VoltageRegulatorCmd(ADC2, ENABLE);
	chThdSleep(MS2ST(4));

	ADC_SelectCalibrationMode(ADC1, ADC_CalibrationMode_Single);
	ADC_SelectCalibrationMode(ADC2, ADC_CalibrationMode_Single);
	ADC_StartCalibration(ADC1);
	ADC_StartCalibration(ADC2);
	chThdSleep(MS2ST(4));

	supply.adcCalSupply = ADC_GetCalibrationValue(ADC1);
	supply.adcCalIr = ADC_GetCalibrationValue(ADC2);

	ADC_StructInit(&adcInit);
	adcInit.ADC_ContinuousConvMode = ADC_ContinuousConvMode_Disable;
	adcInit.ADC_Resolution = ADC_Resolution_12b;
	adcInit.ADC_ExternalTrigConvEvent = ADC_ExternalTrigConvEvent_4;	// TIM3_TRGO event
	adcInit.ADC_ExternalTrigEventEdge = ADC_ExternalTrigEventEdge_RisingEdge;
	adcInit.ADC_DataAlign = ADC_DataAlign_Right;
	adcInit.ADC_OverrunMode = ADC_OverrunMode_Disable;
	adcInit.ADC_AutoInjMode = ADC_AutoInjec_Disable;
	adcInit.ADC_NbrOfRegChannel = 2*NUM_DATA_SAMPLES;
	ADC_Init(ADC1, &adcInit);

	adcInit.ADC_ContinuousConvMode = ADC_ContinuousConvMode_Disable;
	adcInit.ADC_ExternalTrigConvEvent = ADC_ExternalTrigConvEvent_4;	// TIM3_TRGO event
	adcInit.ADC_ExternalTrigEventEdge = ADC_ExternalTrigEventEdge_RisingEdge;
	adcInit.ADC_NbrOfRegChannel = NUM_IR_SAMPLES;
	ADC_Init(ADC2, &adcInit);

	// ADC1 regular channel configuration
	for(uint8_t i = 1; i <= NUM_DATA_SAMPLES; i++)
		ADC_RegularChannelConfig(ADC1, ADC_Channel_3, i, ADC_SampleTime_181Cycles5);	// battery voltage
	for(uint8_t i = NUM_DATA_SAMPLES+1; i <= 2*NUM_DATA_SAMPLES; i++)
		ADC_RegularChannelConfig(ADC1, ADC_Channel_4, i, ADC_SampleTime_181Cycles5);	// current consumption

	// ADC2 regular channel configuration
	for(uint8_t i = 1; i <= NUM_IR_SAMPLES; i++)
		ADC_RegularChannelConfig(ADC2, ADC_Channel_12, i, ADC_SampleTime_181Cycles5);	// IR LED

	// Configures the ADC DMA
	ADC_DMAConfig(ADC1, ADC_DMAMode_Circular);
	ADC_DMAConfig(ADC2, ADC_DMAMode_Circular);
	// Enable the ADC DMA
	ADC_DMACmd(ADC1, ENABLE);
	ADC_DMACmd(ADC2, ENABLE);

	// Enable ADC1
	ADC_Cmd(ADC1, ENABLE);
	ADC_Cmd(ADC2, ENABLE);
	chThdSleep(MS2ST(4));

	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_SUPPLY_MEAS;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	nvicInit.NVIC_IRQChannel = DMA1_Channel1_IRQn;
	NVIC_Init(&nvicInit);

	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_IR_BARRIER;
	nvicInit.NVIC_IRQChannel = DMA2_Channel1_IRQn;
	NVIC_Init(&nvicInit);

	// IR Timer
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	timeBaseInit.TIM_ClockDivision = 0;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 249;	// = 2kHz, rise/fall time: 200us, sample time: 50us
	// ADC 181.5 cycles, 16 samples ~= 44us
	timeBaseInit.TIM_Prescaler = 71;	// = 1MHz
	timeBaseInit.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM3, &timeBaseInit);

	ocInit.TIM_Pulse = 1;
	ocInit.TIM_OCMode = TIM_OCMode_Toggle;
	ocInit.TIM_OutputState = TIM_OutputState_Enable;
	ocInit.TIM_OutputNState = TIM_OutputNState_Disable;
	ocInit.TIM_OCPolarity = TIM_OCPolarity_High;
	ocInit.TIM_OCNPolarity = TIM_OCNPolarity_High;
	ocInit.TIM_OCIdleState = TIM_OCIdleState_Set;
	ocInit.TIM_OCNIdleState = TIM_OCNIdleState_Reset;
	TIM_OC4Init(TIM3, &ocInit);

	ocInit.TIM_Pulse = 199;
	ocInit.TIM_OCMode = TIM_OCMode_PWM2;
	TIM_OC3Init(TIM3, &ocInit);

	TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_OC3Ref);	// OC3 connected to ADC

	TIM_Cmd(TIM3, ENABLE);

	ADC_StartConversion(ADC1);
	ADC_StartConversion(ADC2);

	DMA_Cmd(DMA1_Channel1, ENABLE);
	DMA_Cmd(DMA2_Channel1, ENABLE);

	gpioInit.GPIO_Pin = GPIO_Pin_1;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOB, &gpioInit);

	GPIO_PinAFConfig(GPIOB, GPIO_PinSource1, GPIO_AF_2);
}

static void eventSink(void* pUserData, uint16_t event)
{
	(void)pUserData;
	(void)event;
}

static void periodic(void* pUserData)
{
	(void)pUserData;

	SupplyPowerLog powerLog;
	powerLog.cur = supply.shuntResult*ADC_SHUNT_TO_CURRENT;
	powerLog.bat = supply.batResult*ADC_BAT_TO_VOLTAGE;

	IntercomHeader icHeader;
	icHeader.address = INTERCOM_ADDRESS_MAIN | INTERCOM_ADDRESS_MEDIA;
	icHeader.section = INTERCOM_SECTION_SUPPLY;
	icHeader.command = IC_SUPPLY_POWER_LOG;

	IntercomSend(&intercom1, &icHeader, sizeof(SupplyPowerLog), (uint8_t*)&powerLog);

	IrLog irLog;
	irLog.level[0] = supply.irResult[0];
	irLog.level[1] = supply.irResult[1];

	icHeader.section = INTERCOM_SECTION_KD;
	icHeader.command = IC_KD_IR_LOG;

	IntercomSend(&intercom1, &icHeader, sizeof(IrLog), (uint8_t*)&irLog);
}
