/*
 * ctrl_fusion.h
 *
 *  Created on: 18.03.2013
 *      Author: AndreR
 */

#ifndef CTRL_FUSION_H_
#define CTRL_FUSION_H_

#include "ctrl.h"
#include "util/kf.h"
#include "util/hermite_spline.h"
#include "commands.h"

#define CONTROLLER_MODE_SPLINE 0
#define CONTROLLER_MODE_VEL 1
#define CONTROLLER_MODE_POS 2

#define SENSOR_ACTIVE_VISION (uint8_t) 0x01
#define SENSOR_ACTIVE_ENCODER (uint8_t) 0x02
#define SENSOR_ACTIVE_GYR (uint8_t) 0x04
#define SENSOR_ACTIVE_MOTOR (uint8_t) 0x08
#define SENSOR_ACTIVE_ACC (uint8_t) 0x10

typedef struct _CtrlFusion
{
	KF kfX;
	KF kfY;
	KF kfW;

	uint32_t lastWakeTime;
	uint32_t lastReaction;

	float outMotor[4];
	arm_matrix_instance_f32 matOutMotor;

	float curMotor[4];
	arm_matrix_instance_f32 matCurMotor;

	float curXYW[3];
	arm_matrix_instance_f32 matCurXYW;

	float globalOutXYW[3];
	arm_matrix_instance_f32 matGlobalOutXYW;

	float outXYW[3];
	arm_matrix_instance_f32 matOutXYW;

	float curForce[4];
	arm_matrix_instance_f32 matCurForce;

	float curAcc[3];
	arm_matrix_instance_f32 matCurAcc;

	HermiteSplineList splineList[3];
	float trajectoryStartTimeXY; // [s]
	float trajectoryStartTimeW; // [s]

	float lastPositionUpdateTime;
	float visionTimeout; // [s]
	uint8_t visionLost;
	uint8_t visionBack;
	uint32_t visionBackTime;

	uint8_t activeSensors;

	arm_pid_instance_f32 pidSpline[3];
	arm_pid_instance_f32 pidVel[3];
	arm_pid_instance_f32 pidPos[3];
	arm_pid_instance_f32 pidMotor[4];

	arm_biquad_casd_df1_inst_f32 pt1[3];
	float pt1State[3][4];
	float pt1Coeff[5];

	uint32_t posCounter;
	uint32_t velCounter;

	float posOutput[3];
	float velOutput[3];

	float setpoint[3];
	float theta[4];
} CtrlFusion;

extern CtrlFusion ctrlFusion;
extern CtrlInstance ctrlFusionInstance;

void CtrlFusionSetStateUncertainty(float* pEx);
void CtrlFusionSetSensorUncertainty(float* pEz);
void CtrlFusionAddSplineCmdXY(CtrlSpline2D* pSpline);
void CtrlFusionAddSplineCmdW(CtrlSpline1D* pSpline);
void CtrlFusionSetFilterParams(CtrlSetFilterParams* pParams);
void CtrlFusionSetPIDParams(CtrlSetPIDParams* pParams);
void CtrlFusionSetVelTarget(float x, float y, float w);
void CtrlFusionSetPosTarget(float x, float y, float w);
void CtrlFusionSetPT1TimeConstant(float T);
void CtrlFusionReset();

#endif /* CTRL_FUSION_H_ */
