/*
 * ctrl_fusion_vel.h
 *
 *  Created on: 06.05.2013
 *      Author: AndreR
 */

#ifndef CTRL_FUSION_VEL_H_
#define CTRL_FUSION_VEL_H_

#ifdef SIMULATION
#include "ctrl_sim.h"
#else
#include "ctrl.h"
#endif

#include "util/kf.h"
#include "commands.h"

#define CTRL_FUSION_VEL_NUM_SAMPLES 1000

#define CTRL_FUSION_VEL_LOG_IDLE	0
#define CTRL_FUSION_VEL_LOG_ENC		1
#define CTRL_FUSION_VEL_LOG_ACC		2
#define CTRL_FUSION_VEL_LOG_GYR		3
#define CTRL_FUSION_VEL_LOG_MOTOR	4
#define CTRL_FUSION_VEL_LOG_STEP	5

typedef struct _CtrlFusionVel
{
	KF kfX;
	KF kfY;
	KF kfW;

	uint32_t lastWakeTime;
	uint32_t lastReaction;

	float outMotor[4];
	arm_matrix_instance_f32 matOutMotor;

	float curMotor[4];
	arm_matrix_instance_f32 matCurMotor;

	float curXYW[3];
	arm_matrix_instance_f32 matCurXYW;

	float outXYW[3];
	arm_matrix_instance_f32 matOutXYW;

	float curForce[4];
	arm_matrix_instance_f32 matCurForce;

	float curAcc[3];
	arm_matrix_instance_f32 matCurAcc;

	arm_pid_instance_f32 pidVel[3];

	arm_biquad_casd_df1_inst_f32 pt1[3];
	float pt1State[3][4];
	float pt1Coeff[5];
	arm_biquad_casd_df1_inst_f32 pt1Dt[3];
	float pt1DtState[3][4];
	float pt1DtCoeff[5];

	uint32_t velCounter;

	float velOutput[3];

	float setpoint[3];

	// testing
	uint8_t lastLogState;
	uint8_t logState;
	float testOutput;
	float samples[CTRL_FUSION_VEL_NUM_SAMPLES*4];
	uint32_t numSamples;
} CtrlFusionVel;

extern CtrlFusionVel ctrlFusionVel;
extern CtrlInstance ctrlFusionVelInstance;

void CtrlFusionVelSetVelTarget(float x, float y, float w);
void CtrlFusionVelSetStateUncertainty(float* pEx);
void CtrlFusionVelSetSensorUncertainty(float* pEz);
void CtrlFusionVelSetFilterParams(CtrlSetFilterParams* pParams);
void CtrlFusionVelSetPIDParams(CtrlSetPIDParams* pParams);
void CtrlFusionVelSetPT1TimeConstant(float T);
void CtrlFusionVelTestStep(float step);
void CtrlFusionVelGatherData();
void CtrlFusionVelNetPrintInput();
void CtrlFusionVelEnableLog(uint8_t logType);

#endif /* CTRL_FUSION_H_ */
