/*
 * ctrl_motor.c
 *
 *  Created on: 08.03.2013
 *      Author: AndreR
 */

#include "ctrl_motor.h"
#include "commands.h"
#include "network.h"

#include <string.h>

static void init();
static void controller(CtrlInput* pInput, CtrlOutput* pOutput);

CtrlMotor ctrlMotor;
CtrlInstance ctrlMotorInstance = {.pInit = &init, .pExit = 0, .pControllerFunction = &controller, .deltaT = 0.01f};

static void init()
{
	ctrlMotor.lastWakeTime = chTimeNow();
	ctrlMotor.lastStatusTime = ctrlMotor.lastWakeTime;

	for(uint8_t i = 0; i < 4; i++)
	{
		ctrlMotor.lastReaction[i] = ctrlMotor.lastWakeTime;
	}

	memset(ctrlMotor.setpoint, 0, sizeof(float)*4);

	ctrlMotor.matCurMotor.pData = ctrlMotor.curWheel;
	ctrlMotor.matCurMotor.numCols = 1;
	ctrlMotor.matCurMotor.numRows = 4;

	ctrlMotor.matSetpoint.pData = ctrlMotor.setpoint;
	ctrlMotor.matSetpoint.numCols = 1;
	ctrlMotor.matSetpoint.numRows = 4;
}

static void controller(CtrlInput* pInput, CtrlOutput* pOutput)
{
	if(pInput->enc.updated == 0)
		return;

	pInput->enc.updated = 0;

	memcpy(ctrlMotor.curWheel, pInput->enc.motorVel, sizeof(float)*4);

	ctrlMotor.curWheel[0] *= ctrl.botParams.wheelRadius*CTRL_MOTOR_TO_WHEEL_RATIO;
	ctrlMotor.curWheel[1] *= ctrl.botParams.wheelRadius*CTRL_MOTOR_TO_WHEEL_RATIO;
	ctrlMotor.curWheel[2] *= ctrl.botParams.wheelRadius*CTRL_MOTOR_TO_WHEEL_RATIO;
	ctrlMotor.curWheel[3] *= ctrl.botParams.wheelRadius*CTRL_MOTOR_TO_WHEEL_RATIO; // value is now speed over ground [m/s]

	// velocity state update
	arm_mat_mult_f32(&ctrl.matMotor2XYW, &ctrlMotor.matCurMotor, &ctrl.state.matVel);
	ctrl.state.updated[1] = 1;
	if(pInput->pos.updated)
	{
		pInput->pos.updated = 0;
		memcpy(ctrl.state.pos, pInput->pos.pos, sizeof(float)*3);
		ctrl.state.updated[0] = 1;
	}

	// convert speed over ground to motor output
	for(uint8_t i = 0; i < 4; i++)
	{
		pOutput->motorRPS[i] = ctrlMotor.setpoint[i]*CTRL_WHEEL_TO_MOTOR_RATIO * 1/ctrl.botParams.wheelRadius * 1/(2*PI);
	}

	// logging
	if(ctrlMotor.lastWakeTime - ctrlMotor.lastStatusTime > MS2ST(50))
	{
		ctrlMotor.lastStatusTime = ctrlMotor.lastWakeTime;

		if(ctrlMotor.logEnabled)
		{

			SystemStatusMovement status;
			status.vx = ctrl.state.vel[0];
			status.vy = ctrl.state.vel[1];
			status.vw = ctrl.state.vel[2];

			PacketHeader header;
			header.section = SECTION_SYSTEM;
			header.cmd = CMD_SYSTEM_STATUS_MOVEMENT;

			NetworkSendPacket(&header, (uint8_t*)&status, sizeof(SystemStatusMovement));
		}

		for(uint8_t i = 0; i < 4; i++)
		{
			if(!ctrlMotor.motorLog[i])
				continue;

			MotorPidLog log;
			log.latest = ctrlMotor.curWheel[i]*1000.0f;
			log.setpoint = ctrlMotor.setpoint[i]*1000.0f;
			log.pError = 0;
			log.iError = 0;
			log.dError = 0;
			log.id = i;
			log.output = pOutput->motorRPS[i]*1000.0f;
			log.overload = 0;
			log.eCurrent = 0;

			PacketHeader header;
			header.section = SECTION_MOTOR;
			header.cmd = CMD_MOTOR_PID_LOG;

			NetworkSendPacket(&header, (uint8_t*)&log, sizeof(MotorPidLog));
		}
	}

	ctrlMotor.lastWakeTime = chTimeNow();
}

void CtrlMotorSetTarget(float x, float y, float w)
{
	float xyw[3] = {x, y , w};
	arm_matrix_instance_f32 matXYW = {.numCols = 1, .numRows = 3, .pData = xyw};

	arm_mat_mult_f32(&ctrl.matXYW2Motor, &matXYW, &ctrlMotor.matSetpoint);
}

void CtrlMotorSetMotorLog(uint8_t id, uint8_t enable)
{
	ctrlMotor.motorLog[id] = enable;
}

void CtrlMotorSetLog(uint8_t enable)
{
	ctrlMotor.logEnabled = enable;
}
