/*
 * exti.c
 *
 *  Created on: 29.05.2010
 *      Author: AndreR
 */

#include "exti.h"

#include "stm32f4xx.h"
#include "misc.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_exti.h"
#include "stm32f4xx_gpio.h"

#include "constants.h"
#include "ch.h"

Exti exti;

void EXTI9_5_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	for(uint8_t i = 5; i <= 9; i++)
	{
		if(EXTI->PR & (1 << i))
		{
			if(exti.callbacks[i])
				(*exti.callbacks[i])();

			EXTI->PR = (1 << i);
		}
	}

	CH_IRQ_EPILOGUE();
}

void EXTI15_10_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	for(uint8_t i = 10; i <= 15; i++)
	{
		if(EXTI->PR & (1 << i))
		{
			if(exti.callbacks[i])
				(*exti.callbacks[i])();

			EXTI->PR = (1 << i);
		}
	}

	CH_IRQ_EPILOGUE();
}

void ExtiInit()
{
	for(uint8_t i = 0; i < 16; i++)
		exti.callbacks[i] = 0;

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = EXTI9_5_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_EXTI5_9;	//lowest priority
	NVICInit.NVIC_IRQChannelSubPriority = 0;	//lowest priority
	NVIC_Init(&NVICInit);

	NVICInit.NVIC_IRQChannel = EXTI15_10_IRQn;	// interrupts must be configured separately
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_EXTI10_15;	//lowest priority
	NVIC_Init(&NVICInit);
}

void ExtiSetCallback(ExtiCallback cb, uint8_t channel)
{
	exti.callbacks[channel] = cb;
}
