/*
 * usart2.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart2.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "constants.h"

USART usart2;

void USART2_IRQHandler()
{
	USARTIRQ(&usart2);
}

void DMA1_Stream5_IRQHandler()
{
//	USARTDMARx(&usart2);
}

// USART2 TX
void DMA1_Stream6_IRQHandler()
{
//	USARTDMATx(&usart2);
}

void TIM2_IRQHandler()
{
	TIM2->SR = 0;
	TIM_Cmd(TIM2, DISABLE);

	USARTTimerIRQ(&usart2);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM2->ARR = microSeconds;

	TIM_Cmd(TIM2, ENABLE);
}

void USART2Init()
{
	// TIM2 setup
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1000;	// = 1ms
	timeBaseInit.TIM_Prescaler = 83; // clock is now 1MHz
	TIM_TimeBaseInit(TIM2, &timeBaseInit);

	TIM2->SR = 0;
	TIM_ARRPreloadConfig(TIM2, DISABLE);
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM2_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_USART2;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	// enable peripheral clock
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

	// provide data storage
	static uint8_t usart2TxData[USART2_FIFO_SIZE];
	static uint8_t usart2RxData[USART2_FIFO_SIZE];

	// map data and configure
	USARTData data;
	data.pDataRx = usart2RxData;
	data.rxSize = USART2_FIFO_SIZE;
	data.pDataTx = usart2TxData;
	data.txSize = USART2_FIFO_SIZE;
	data.maxPacketSize = USART2_MAX_PACKET_SIZE;
	data.pDMA = DMA1;
	data.dmaChannel = DMA_Channel_4;
	data.rxStream = 5;
	data.txStream = 6;
	data.rxIRQn = DMA1_Stream5_IRQn;
	data.txIRQn = DMA1_Stream6_IRQn;
	data.usartIRQn = USART2_IRQn;
	data.IRQL = IRQL_USART2;
	data.baudrate = 3000000;
	data.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart2, USART2, data);

	// GPIO setup
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOA, &gpioInit);

	// Map alternate function
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_USART2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_USART2);
}
