/*
 * usart6.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart6.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "constants.h"

USART usart6;

void USART6_IRQHandler()
{
	USARTIRQ(&usart6);
}

void DMA2_Stream1_IRQHandler()
{
//	USARTDMARx(&usart6);
}

// USART2 TX
void DMA2_Stream6_IRQHandler()
{
//	USARTDMATx(&usart6);
}

void TIM1_TRG_COM_TIM11_IRQHandler()
{
	TIM11->SR = 0;
	TIM_Cmd(TIM11, DISABLE);

	USARTTimerIRQ(&usart6);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM11->ARR = microSeconds;

	TIM_Cmd(TIM11, ENABLE);
}

void USART6Init()
{
	// TIM11 setup
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM11, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1000;	// = 1ms
	timeBaseInit.TIM_Prescaler = 83; // clock is now 1MHz
	TIM_TimeBaseInit(TIM11, &timeBaseInit);

	TIM11->SR = 0;
	TIM_ARRPreloadConfig(TIM11, DISABLE);
	TIM_ITConfig(TIM11, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM1_TRG_COM_TIM11_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_USART6;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	// enable peripheral clock
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART6, ENABLE);

	// provide data storage
	static uint8_t usart6TxData[USART6_FIFO_SIZE];
	static uint8_t usart6RxData[USART6_FIFO_SIZE];

	// map data and configure
	USARTData data;
	data.pDataRx = usart6RxData;
	data.rxSize = USART6_FIFO_SIZE;
	data.pDataTx = usart6TxData;
	data.txSize = USART6_FIFO_SIZE;
	data.maxPacketSize = USART6_MAX_PACKET_SIZE;
	data.pDMA = DMA2;
	data.dmaChannel = DMA_Channel_5;
	data.rxStream = 1;
	data.txStream = 6;
	data.rxIRQn = DMA2_Stream1_IRQn;
	data.txIRQn = DMA2_Stream6_IRQn;
	data.usartIRQn = USART6_IRQn;
	data.IRQL = IRQL_USART6;
	data.baudrate = 5250000;
	data.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart6, USART6, data);

	// GPIO setup
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOC, &gpioInit);

	// Map alternate function
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_USART6);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_USART6);
}
