/*
 * dect.h
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#ifndef __NETWORK_H__
#define __NETWORK_H__

#include "util/nrf24_ex.h"
#include "util/flash_fs.h"

#include "constants.h"
#include "commands.h"
#include "intercom_constants.h"

#include "ch.h"

typedef void(*PosCallback)();
typedef void(*TrackedObjCallback)();

typedef struct _Network
{
	uint8_t txProcessingBuf[NETWORK_TX_PROCESSING_SIZE];
	uint8_t rxProcessingBuf[NETWORK_RX_PROCESSING_SIZE];

	uint8_t netPrint[NETWORK_PRINT_SIZE];

	BinarySemaphore timerSem;
	Mutex sendPacketMutex;
	Mutex printMutex;

	NRF24Queue queue;
	NRF24Ex nrf24;

	uint32_t lastUSec;
	uint32_t avgRxPeriod;

	PosCallback posCb;
	TrackedObjCallback trackedObjCb;

	int16_t lastVisionPos[3];
	int16_t lastTrackedObjPos[3];

	ConfigNetwork config;
	FlashFile* pConfigFile;
} Network;

extern Network network;

void	NetworkInit();
msg_t	NetworkTask(void* params);
int16_t NetworkSendPacket(PacketHeader* pHeader, uint8_t* pData, uint16_t dataLength);
void	NetworkSetPosCallback(PosCallback cb);
void	NetworkSetTrackedObjCallback(TrackedObjCallback cb);
int32_t NetworkPrint(const char* fmt, ...);
void	NetworkSaveConfig();

#endif /* DECT_H_ */
