/*
 * spi2.c
 *
 *  Created on: 19.01.2014
 *      Author: AndreR
 */

#include "spi2.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_tim.h"
#include "misc.h"

#include "constants.h"

#include "util/console.h"

SPI2Global spi2;

#define EVENT_TRANSFER_MASK	0x01FF

#define EVENT_DMA_TX_TE		0x0001
#define EVENT_DMA_TX_DME	0x0002
#define EVENT_DMA_TX_FE		0x0004
#define EVENT_DMA_TX_TC		0x0008
#define EVENT_DMA_RX_TE		0x0010
#define EVENT_DMA_RX_DME	0x0020
#define EVENT_DMA_RX_FE		0x0040
#define EVENT_DMA_RX_TC		0x0080
#define EVENT_SPI_CRCERR	0x0100
#define EVENT_TIMER			0x1000

#define STATE_IDLE			0
#define STATE_ACTIVE		1

#define DMA_Stream_RX		DMA1_Stream3
#define DMA_Stream_TX		DMA1_Stream4

void DMA1_Stream3_IRQHandler()
{
	SPISyncDmaRxInterrupt(&spi2.slave[spi2.activeSlave]);
}

// TODO: replace timer
//void TIM8_UP_TIM13_IRQHandler()
//{
//	TIM13->SR = 0;
//
//	uint16_t event = EVENT_TIMER;
//	int32_t yield = 0;
//	xQueueSendFromISR(spi2.hEventQueue, &event, &yield);
//	if(yield)
//		taskYIELD();
//}

void SPI2Init()
{
	// init variables
	spi2.activeSlave = ACC;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

	SPISyncData spiData;
	spiData.cpha = 0;
	spiData.cpol = 0;
	spiData.csPin = GPIO_Pin_10;
	spiData.pCSPort = GPIOB;
	spiData.dmaChRx = 3;
	spiData.dmaChTx = 4;
	spiData.dmaPrio = 1;
	spiData.pDma = DMA1;
	spiData.pRegister = SPI2;
	spiData.prescaler = SPI_BaudRatePrescaler_16;
	spiData.timeoutTicks = MS2ST(100);
	SPISyncInit(&spi2.slave[GYR], &spiData);

	spiData.csPin = GPIO_Pin_12;
	SPISyncInit(&spi2.slave[ACC], &spiData);


#ifdef DEBUG
	spi2.pRxDma = DMA_Stream_RX;
	spi2.pTxDma = DMA_Stream_TX;
	spi2.pSPI = SPI2;
	spi2.pDMA = DMA1;
#endif

	// init GPIO
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_25MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	gpioInit.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;	// SCK, MISO, MOSI
	GPIO_Init(GPIOB, &gpioInit);

	GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_SPI2);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_SPI2);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_SPI2);

	// configure NVIC
	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = DMA1_Stream3_IRQn;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI2;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&nvicInit);

    // init and start timer
/*	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM13, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 500;	// = 1ms
	timeBaseInit.TIM_Prescaler = 83; // clock is now 1MHz
	TIM_TimeBaseInit(TIM13, &timeBaseInit);

	TIM13->SR = 0;
	TIM_ARRPreloadConfig(TIM13, DISABLE);
	TIM_ITConfig(TIM13, TIM_IT_Update, ENABLE);

	nvicInit.NVIC_IRQChannel = TIM8_UP_TIM13_IRQn;
	NVIC_Init(&nvicInit);*/
}

msg_t SPI2Task(void* params)
{
	(void)params;

	chRegSetThreadName("SPI2");

	int16_t result;
	int16_t x, y, z;
	static uint8_t tx[7];
	static uint8_t rx[7];

//	xQueueReset(spi2.hEventQueue);
//	TIM_Cmd(TIM13, ENABLE);

	spi2.activeSlave = ACC;

	tx[0] = 0x00 | 0x20;
	tx[1] = 0x87;
	result = SPISyncTransfer(&spi2.slave[ACC], tx, rx, 2);

	tx[0] = 0x00 | 0x40 | 0x24;
	tx[1] = 0x14;	// mag @ 100Hz
	tx[2] = 0;
	tx[3] = 0;
	result = SPISyncTransfer(&spi2.slave[ACC], tx, rx, 4);

	spi2.activeSlave = GYR;

	tx[0] = 0x20 | 0x40; // CTRL1 Register + multiple write bit
	tx[1] = 0xBF; // CTRL1: 400Hz, Cut-off 110Hz, X,Y,Z on
	tx[2] = 0x09; // CTRL2: High Pass 0.05Hz
	tx[3] = 0x00; // CTRL3: Data Ready interrupt
	tx[4] = 0x20; // CTRL4: FSS@2000dps
	result = SPISyncTransfer(&spi2.slave[GYR], tx, rx, 5);

	(void)result;

	while(1)
	{
		// acc
		spi2.activeSlave = ACC;

		tx[0] = 0x80 | 0x40 | 0x28;
		SPISyncTransfer(&spi2.slave[ACC], tx, rx, 7);

		x = *((int16_t*)(rx+1));
		y = *((int16_t*)(rx+3));
		z = *((int16_t*)(rx+5));

//		ConsolePrint("ACC X: %5hd, Y: %5hd, Z: %5hd", x, y, z);

		// gyr
		spi2.activeSlave = GYR;

		tx[0] = 0x80 | 0x40 | 0x28;
		SPISyncTransfer(&spi2.slave[GYR], tx, rx, 7);

		x = *((int16_t*)(rx+1));
		y = *((int16_t*)(rx+3));
		z = *((int16_t*)(rx+5));

//		ConsolePrint("\t\tGYR X: %5hd, Y: %5hd, Z: %5hd\r\n", x, y, z);

		(void)x;
		(void)y;
		(void)z;

		chThdSleep(MS2ST(100));
	}

	return 0;
}
