/*
 * ui.c
 *
 *  Created on: 22.05.2014
 *      Author: AndreR
 */

#include "ui.h"
#include "intercom1.h"
#include "intercom_constants.h"
#include "network.h"

#define UI_EVENT_QUEUE_SIZE 4

typedef struct _UI
{
	Mailbox eventQueue;
	msg_t eventQueueData[UI_EVENT_QUEUE_SIZE];
} UI;

UI ui;

static void uiCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case IC_UI_WIFI_BOTID:
		{
			UIWifiBotID* pID = (UIWifiBotID*)pData;

			NRF24ExChangeId(&network.nrf24, pID->id);
			network.config.botId = pID->id;
		}
		break;
		case IC_UI_WIFI_SAVE:
		{
			NetworkSaveConfig();
		}
		break;
	}
}

void UIInit()
{
	chMBInit(&ui.eventQueue, ui.eventQueueData, UI_EVENT_QUEUE_SIZE);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &uiCallback;
	client.section = INTERCOM_SECTION_UI;

	IntercomAddClient(&intercom1, client);
}

msg_t UITask(void* params)
{
	(void)params;
	msg_t event;

	while(1)
	{
		chMBFetch(&ui.eventQueue, &event, MS2ST(500));

		IntercomHeader iHeader;
		iHeader.address = INTERCOM_ADDRESS_MEDIA;
		iHeader.section = INTERCOM_SECTION_PRESENTER;
		iHeader.command = IC_PRESENTER_CONFIG;

		IntercomSend(&intercom1, &iHeader, sizeof(ConfigNetwork), (uint8_t*)&network.config);
	}

	return 0;
}
