/*
 * cli.c
 *
 *  Created on: 19.11.2012
 *      Author: AndreR
 */

#include "cli.h"

#include "util/console.h"
#include "util/log.h"
#include "intercom_constants.h"
#include "hal/led.h"
#include "hal/usart2.h"
#include "hal/sram.h"
#include "hal/tft.h"
#include "intercom1.h"
#include "GUI.h"
#include "util/sys_time.h"
#include "ff.h"

CLI cli;

#define SRAM_TEST_SIZE 512*1024
static volatile EXT_SRAM uint16_t sram[SRAM_TEST_SIZE];
static FATFS fatfs;

void processCommand();

void CLIInit()
{
	chMtxInit(&cli.procBufMutex);

	static uint8_t rxData[CONSOLE_RX_SIZE];
	static uint8_t txData[CONSOLE_TX_SIZE];

	ConsoleData conData;
	conData.pDataTx = txData;
	conData.txSize = CONSOLE_TX_SIZE;
	conData.pDataRx = rxData;
	conData.rxSize = CONSOLE_RX_SIZE;
	conData.printSize = CONSOLE_PRINT_SIZE;
	conData.txCallback = 0;
	conData.pUserData = 0;

	ConsoleInit(conData);

	ConsoleIntercomAdapterInit(&cli.adapter, &intercom1, INTERCOM_ADDRESS_MEDIA, INTERCOM_ADDRESS_MAIN);
}

msg_t CLITask(void* params)
{
	(void)params;

	systime_t waitUntil = chTimeNow()+MS2ST(10);
	systime_t lastLEDTime = 0;

	LEDMustOff(LED_MAIN_GREEN | LED_MAIN_RED);

	while(1)
	{
		if(ConsoleIsCmdReady())
		{
			processCommand();

			ConsoleCmdProcessed();
		}

		// Heartbeat LED
		if(chTimeElapsedSince(lastLEDTime) > MS2ST(500))
		{
			lastLEDTime = chTimeNow();

#ifdef ENV_BOOT
			LEDToggle(LED_MAIN_RED);
#else
			LEDToggle(LED_MAIN_GREEN);
#endif
		}

		chThdSleepUntil(waitUntil);
		waitUntil = chTimeNow()+MS2ST(10);
	}

	return 0;
}

void processCommand()
{
	static const uint16_t bufLen = 128;
	static uint8_t buf8[128];
	static uint32_t buf32[10];

	if(ConsoleCmpCmd("test"))
	{
		ConsolePrint("Number: %d\r\n", 43);
	}

	if(ConsoleCmpCmd("test2"))
	{
		ConsolePrint("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789\r\n");
	}

/*	if(ConsoleCmpCmd("task list"))
	{
		uint8_t* pPrintBuf;

		ConsolePrint("\r\nTask\t\tState\tPrio\tStack\tID");

		if(ConsoleLockPrintBuf(400, &pPrintBuf) == 0)
		{
			vTaskList((signed char*)pPrintBuf);

			ConsoleUnlockPrintBuf()
		}


		ConsoleLockPrintBuf();
		vTaskList((signed char*)printBuf);
		ConsoleSendPrintBuf();
		ConsoleUnlockPrintBuf();
	}

	if(ConsoleCmpCmd("task stats"))
	{
		char* printBuf = ConsoleGetPrintBuf();

		ConsolePrint("\r\nTask\t\tTicks\t\tRatio");

		ConsoleLockPrintBuf();
		vTaskGetRunTimeStats((signed char*)printBuf);
		ConsoleSendPrintBuf();
		ConsoleUnlockPrintBuf();
	}
*/
	if(ConsoleCmpCmd("usart2 state"))
	{
		ConsolePrint("TX: %d\r\n", usart2.txState);
	}

	if(ConsoleCmpCmd("time"))
	{
		ConsolePrint("Time: %uus\r\n", SysTimeUSec());
	}

	if(ConsoleScanCmd("tft brightness %d", buf32) == 1)
	{
		ConsolePrint("TFT brightness set to: %d\r\n");
		TFTSetBrightness((uint8_t)buf32[0]);
	}

	FRESULT fresult;

	if(ConsoleCmpCmd("mount"))
	{
		fresult = f_mount(&fatfs, "0:/", 1);
		ConsolePrint("Mount: %u\r\n", fresult);
	}

	if(ConsoleCmpCmd("pwd"))
	{
		f_getcwd((TCHAR*)buf8, bufLen);

		ConsolePrint("%s\r\n", buf8);
	}

	if(ConsoleCmpCmd("ls"))
	{
		DIR dir;
		FILINFO finfo;
		finfo.lfname = (TCHAR*)buf8;
		finfo.lfsize = bufLen-1;

		f_getcwd((TCHAR*)buf8, bufLen);

		ConsolePrint("Content of: %s\r\n", buf8);

		fresult = f_opendir(&dir, "");
		if(fresult != FR_OK)
			return;

		do
		{
			fresult = f_readdir(&dir, &finfo);

			if(finfo.fname[0] == 0)
				break;

			if(finfo.fattrib & AM_DIR)
				ConsolePrint("%s/\r\n", *finfo.lfname ? finfo.lfname : finfo.fname);
			else
				ConsolePrint("%s\r\n", *finfo.lfname ? finfo.lfname : finfo.fname);
		}
		while(fresult == FR_OK);
	}

	if(ConsoleScanCmd("cd %127[^\n]", buf8) == 1)
	{
		f_chdir((TCHAR*)buf8);

		f_getcwd((TCHAR*)buf8, bufLen);

		ConsolePrint("%s\r\n", buf8);
	}

	if(ConsoleScanCmd("touch %127[^\n]", buf8) == 1)
	{
		FIL file;
		fresult = f_open(&file, (TCHAR*)buf8, FA_CREATE_NEW);
		if(fresult == FR_EXIST)
			ConsolePrint("File already exists\r\n");
		f_close(&file);
	}

	if(ConsoleScanCmd("mkdir %127[^\n]", buf8) == 1)
	{
		fresult = f_mkdir((TCHAR*)buf8);
		if(fresult == FR_OK)
			ConsolePrint("Created dir: %s\r\n", buf8);
	}

	if(ConsoleScanCmd("rm %127[^\n]", buf8) == 1)
	{
		fresult = f_unlink((TCHAR*)buf8);
		if(fresult != FR_OK)
			ConsolePrint("Could not remove object\r\n");
	}

	if(ConsoleScanCmd("read %u %127[^\n]", buf32, buf8) == 2)
	{
		ConsolePrint("Reading %uB from file: %s\r\n", *buf32, buf8);

		FIL file;
		fresult = f_open(&file, (TCHAR*)buf8, FA_READ);
		if(fresult != FR_OK)
		{
			ConsolePrint("Open file failed\r\n");
			return;
		}

		uint32_t bytesRead;
		uint32_t timeStart = SysTimeUSec();
		fresult = f_read(&file, (void*)sram, *buf32, (UINT*)&bytesRead);
		uint32_t timeEnd = SysTimeUSec();
		ConsolePrint("Result: %d\r\n", fresult);
		ConsolePrint("Bytes read: %u\r\n", bytesRead);
		uint32_t timeUs = timeEnd-timeStart;
		ConsolePrint("Time: %uus\r\n", timeUs);
	}

	if(ConsoleCmpCmd("log"))
	{
		LogPrintAllRecent(10);
	}
}
