/*
 * ov7725.c
 *
 *  Created on: 02.12.2012
 *      Author: AndreR
 */

#include "ov7725.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_dcmi.h"
#include "stm32f4xx_dma.h"
#include "misc.h"

#include "sram.h"

#include "tft.h"

#include "util/console.h"
#include "constants.h"

OV7725 ov7725;

static const uint8_t qvgaReg[][2] =
{
	{0x12, 0x80},

	{0x12, 0x46},
//	{0x3d, 0x03},
//	{0x17, 0x22},
//	{0x18, 0xa4},
//	{0x19, 0x07},
//	{0x1a, 0xf0},
	{0x32, 0x00},
//	{0x29, 0x50},
//	{0x2c, 0x78},
//	{0x2a, 0x00},
	{0x11, 0x01},//00/01/03/07 for 60/30/15/7.5fps

//	{0x42, 0x7f},
//	{0x4d, 0x09},
//	{0x63, 0xe0},
//	{0x64, 0xff},
//	{0x65, 0x2f},
//	{0x66, 0x00},
//	{0x67, 0x48},
//
//	{0x13, 0xf0},
//	{0x0d, 0x41},
//	{0x0f, 0xc5},
//	{0x14, 0x11},
	{0x22, 0x7f},//ff/7f/3f/1f for 60/30/15/7.5fps
	{0x23, 0x03},//01/03/07/0f for 60/30/15/7.5fps
	{0x24, 0x50},
	{0x25, 0x30},
	{0x26, 0xa1},

	{0x6b, 0xaa},
	{0x13, 0xff},

	{0x90, 0x05},
	{0x91, 0x01},
	{0x92, 0x03},
	{0x93, 0x00},

	{0x94, 0x5f},//b0
	{0x95, 0x53},//9d
	{0x96, 0x11},//13
	{0x97, 0x1a},//16
	{0x98, 0x3d},//7b
	{0x99, 0x5a},//91
	{0x9a, 0x1e},//1e

	{0x9b, 0x00},
	{0x9c, 0x25},
	{0x9e, 0x81},
	{0xa6, 0x06},

	//modified saturation initialization value by pw 2008-03-04
	{0xa7, 0x65},
	{0xa8, 0x65},

	{0x7e, 0x0c},
	{0x7f, 0x16},
	{0x80, 0x2a},
	{0x81, 0x4e},
	{0x82, 0x61},
	{0x83, 0x6f},
	{0x84, 0x7b},
	{0x85, 0x86},
	{0x86, 0x8e},
	{0x87, 0x97},
	{0x88, 0xa4},
	{0x89, 0xaf},
	{0x8a, 0xc5},
	{0x8b, 0xd7},
	{0x8c, 0xe8},
	{0x8d, 0x20},
#if 0
	{0x34, 0x00},
	{0x33, 0x40},
	{0x22, 0x99},
	{0x23, 0x03},
	{0x4a, 0x10},
	{0x49, 0x10},
	{0x4b, 0x14},
	{0x4c, 0x17},
	{0x46, 0x05},
	{0x0e, 0x65},
#endif
	{0x69, 0x5d},
	{0x0c, 0x00},

	{0x33, 0x49},//0x66/0x99
	{0x34, 0x01},//0x66/0x99
	{0x0e, 0x65},

	{0x13, 0xff}, //Enable banding Filter

	//sharpness strength modified by pw 2008-03-05
	{0xac, 0xdf},
	{0x8f, 0x04},

	{0xFF, 0x00},
};

static EXT_SRAM uint8_t picBuf[307200]; // sufficient for an 640x480 image

void DMA2_Stream1_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	DMA_ClearITPendingBit(DMA2_Stream1, DMA_IT_TCIF1);

	if(ov7725.mode == OV7725_MODE_STREAM_TO_DISPLAY)
	{
		*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	}

	if(ov7725.mode == OV7725_MODE_SNAPSHOT)
	{
		chSysLockFromIsr();
		chBSemSignalI(&ov7725.acqDoneSem);
		chSysUnlockFromIsr();
	}

	CH_IRQ_EPILOGUE();
}

static void regInit()
{
	for(uint8_t i = 0; 1; i++)
	{
		if(qvgaReg[i][0] == 0xFF) // stop?
			break;

		OV7725WriteReg(qvgaReg[i][0], qvgaReg[i][1]);
	}
}

void OV7725Init()
{
	GPIO_InitTypeDef gpioInit;

	chBSemInit(&ov7725.acqDoneSem, TRUE);

	// Connect DCMI pins to AF13
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI);	// HSYNC
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI); // PCLK
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_DCMI); // D0

	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI);	// D5
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI); // VSYNC

	GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI); // D1
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_DCMI); // D2
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_DCMI); // D3

	GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_DCMI); // D4
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource5, GPIO_AF_DCMI); // D6
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource6, GPIO_AF_DCMI); // D7

	// DCMI GPIO configuration
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_Speed = GPIO_Speed_100MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	gpioInit.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6 | GPIO_Pin_9;
	GPIO_Init(GPIOA, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_Init(GPIOB, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_Init(GPIOC, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;
	GPIO_Init(GPIOE, &gpioInit);

	// FSIN:	A5
	// PWDN:	A7
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_7; // FSIN, PWDN
	GPIO_Init(GPIOA, &gpioInit);

	GPIOA->BSRRH = GPIO_Pin_5 | GPIO_Pin_7;  // PWDN low = normal mode, FSIN low

	// Init control interface
	SCCBInit(&ov7725.sccb, GPIOB, GPIO_Pin_10, GPIO_Pin_11, 0x42); // not joking, 0x42 is really the OV7725 device address

	chThdSleep(MS2ST(100));

	regInit();

	//--- DCMI Configuration ---

	// Enable DCMI clock
	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

	DMA_DeInit(DMA2_Stream1);

	OV7725SetMode(OV7725_MODE_SNAPSHOT);

	DMA_ITConfig(DMA2_Stream1, DMA_IT_TC, ENABLE);

	// NVIC setup
	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannel = DMA2_Stream1_IRQn;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_CAM;
	nvicInit.NVIC_IRQChannelSubPriority = 0x00;
	NVIC_Init(&nvicInit);
}

int16_t OV7725WriteReg(uint16_t addr, uint8_t data)
{
	return SCCBWriteReg(&ov7725.sccb, addr, data);
}

int16_t OV7725ReadReg(uint16_t addr, uint8_t* pData)
{
	return SCCBReadReg(&ov7725.sccb, addr, pData);
}


void OV7725SetMode(uint8_t mode)
{
	DCMI_CaptureCmd(DISABLE);
	DMA_Cmd(DMA2_Stream1, DISABLE);
	DCMI_Cmd(DISABLE);

	ov7725.mode = mode;

	DMA_InitTypeDef dmaInit;
	dmaInit.DMA_Channel = DMA_Channel_1;
	dmaInit.DMA_PeripheralBaseAddr = (uint32_t)&DCMI->DR;
	dmaInit.DMA_DIR = DMA_DIR_PeripheralToMemory;
	dmaInit.DMA_BufferSize = 38400;
	dmaInit.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	dmaInit.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	dmaInit.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	dmaInit.DMA_Priority = DMA_Priority_High;
	dmaInit.DMA_FIFOMode = DMA_FIFOMode_Enable;
	dmaInit.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
	dmaInit.DMA_MemoryBurst = DMA_MemoryBurst_Single;
	dmaInit.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;

	DCMI_InitTypeDef dcmiInit;
	dcmiInit.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
	dcmiInit.DCMI_PCKPolarity = DCMI_PCKPolarity_Rising;
	dcmiInit.DCMI_VSPolarity = DCMI_VSPolarity_High;	// data valid on VSync low
	dcmiInit.DCMI_HSPolarity = DCMI_HSPolarity_Low;		// data valid on HSync high
	dcmiInit.DCMI_CaptureRate = DCMI_CaptureRate_All_Frame;
	dcmiInit.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;

	if(mode == OV7725_MODE_SNAPSHOT)
	{
		dcmiInit.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;
		DCMI_Init(&dcmiInit);

		dmaInit.DMA_Memory0BaseAddr = (uint32_t)picBuf;
		dmaInit.DMA_MemoryInc = DMA_MemoryInc_Enable;
		dmaInit.DMA_Mode = DMA_Mode_Normal;
		DMA_Init(DMA2_Stream1, &dmaInit);
	}

	if(mode == OV7725_MODE_STREAM_TO_DISPLAY)
	{
		dcmiInit.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
		DCMI_Init(&dcmiInit);

		dmaInit.DMA_Memory0BaseAddr = (uint32_t)TFT_DATA_REG;
		dmaInit.DMA_MemoryInc = DMA_MemoryInc_Disable;
		dmaInit.DMA_Mode = DMA_Mode_Circular;
		DMA_Init(DMA2_Stream1, &dmaInit);

		DMA_Cmd(DMA2_Stream1, ENABLE);
		DCMI_Cmd(ENABLE);

		*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;

		DCMI_CaptureCmd(ENABLE);
	}
}

int16_t OV7725AcquireImage()
{
	int16_t result = 0;

	if(ov7725.mode != OV7725_MODE_SNAPSHOT)
		return 2;

	chBSemReset(&ov7725.acqDoneSem, TRUE);

	DMA2_Stream1->NDTR = 38400;

	DMA2->LIFCR = (DMA_LIFCR_CFEIF1 | DMA_LIFCR_CHTIF1 | DMA_LIFCR_CTCIF1 | DMA_LIFCR_CTEIF1);

	DMA_Cmd(DMA2_Stream1, ENABLE);
	DCMI_Cmd(ENABLE);

	DCMI_CaptureCmd(ENABLE);

	if(chBSemWaitTimeout(&ov7725.acqDoneSem, MS2ST(5000)) != RDY_OK)
		result = 1;

	DMA_Cmd(DMA2_Stream1, DISABLE);
	DCMI_Cmd(DISABLE);

	ConsolePrint("Captured %d pixels\r\n", (38400-DMA2_Stream1->NDTR)*2);

	return result;
}

uint8_t* OV7725GetPicBuf()
{
	return picBuf;
}
