/*
 * usart2.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart2.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "constants.h"

USART usart2;

void USART2_IRQHandler()
{
	USARTIRQ(&usart2);
}

void TIM1_TRG_COM_TIM11_IRQHandler()
{
	TIM11->SR = 0;
	TIM_Cmd(TIM11, DISABLE);

	USARTTimerIRQ(&usart2);
}

static void startTimeout(uint32_t microSeconds)
{
	TIM11->ARR = microSeconds;

	TIM_Cmd(TIM11, ENABLE);
}

void USART2Init()
{
	// TIM11 setup
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM11, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1000;	// = 1ms
	timeBaseInit.TIM_Prescaler = 83; // clock is now 1MHz
	TIM_TimeBaseInit(TIM11, &timeBaseInit);

	TIM11->SR = 0;
	TIM_ARRPreloadConfig(TIM11, DISABLE);
	TIM_ITConfig(TIM11, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM1_TRG_COM_TIM11_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_USART2;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	// enable peripheral clock
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

	// provide data storage
	static uint8_t usart2TxData[USART2_FIFO_SIZE];
	static uint8_t usart2RxData[USART2_FIFO_SIZE];

	// map data and configure
	USARTData data2;
	data2.pDataRx = usart2RxData;
	data2.rxSize = USART2_FIFO_SIZE;
	data2.pDataTx = usart2TxData;
	data2.txSize = USART2_FIFO_SIZE;
	data2.maxPacketSize = USART2_MAX_PACKET_SIZE;
	data2.pDMA = DMA1;
	data2.dmaChannel = DMA_Channel_4;
	data2.rxStream = 5;
	data2.txStream = 6;
	data2.rxIRQn = DMA1_Stream5_IRQn;
	data2.txIRQn = DMA1_Stream6_IRQn;
	data2.usartIRQn = USART2_IRQn;
	data2.IRQL = IRQL_USART2;
	data2.baudrate = 5250000;
	data2.pStartTimer = &startTimeout;

	// Init USART util layer
	USARTInit(&usart2, USART2, data2);

	// GPIO setup
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOA, &gpioInit);

	// Map alternate function
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_USART2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_USART2);
}
