/*
 * presenter.c
 *
 *  Created on: 25.03.2014
 *      Author: rYan
 */

#include "presenter.h"

#include "GUI.h"
#include "PROGBAR.h"
#include "BUTTON.h"
#include "DIALOG.h"
#include "EDIT.h"
#include "MENU.h"
#include "FRAMEWIN.h"
#include "util/console.h"

#include "gui/numpad.h"
#include "gui/desktop.h"
#include "gui/select_botid.h"
#include "gui/ids.h"

static void init(void* pUserData);
static void eventSink(void* pUserData, uint16_t event);
static void periodic(void* pUserData);

#include "intercom1.h"
#include "intercom_constants.h"

#define EVENT_POWER_UPDATE	0
#define EVENT_KICKER_UPDATE	1
#define EVENT_IR_UPDATE		2
#define EVENT_CONFIG_UPDATE	3

#define ID_MENU_WIFI_BOTID 		GUI_ID_PRESENTER + 0
#define ID_MENU_WIFI_SAVE		GUI_ID_PRESENTER + 1
#define ID_MENU_WIFI_SETTINGS	GUI_ID_PRESENTER + 2

static struct _Presenter
{
	EventClient* pEventClient;

	SupplyPowerLog lastPowerLog;
	KickerLog lastKickerLog;
	IrLog lastIrLog;
	ConfigNetwork lastConfig;

	WM_HWIN hSelectBotID;

	PROGBAR_Handle hBatBar;
} presenter;

static void supplyCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_SUPPLY_POWER_LOG && dataLength >= sizeof(SupplyPowerLog))
	{
		SupplyPowerLog* pLog = (SupplyPowerLog*)pData;

		presenter.lastPowerLog = *pLog;

		EventClientSend(presenter.pEventClient, EVENT_POWER_UPDATE, 0);
	}
}

static void presenterCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_PRESENTER_CONFIG && dataLength >= sizeof(ConfigNetwork))
	{
		memcpy(&presenter.lastConfig, pData, sizeof(ConfigNetwork));

		EventClientSend(presenter.pEventClient, EVENT_CONFIG_UPDATE, 0);
	}
}

static void kdCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_KD_KICKER_LOG && dataLength >= sizeof(KickerLog))
	{
		KickerLog* pLog = (KickerLog*)pData;

		presenter.lastKickerLog = *pLog;

		EventClientSend(presenter.pEventClient, EVENT_KICKER_UPDATE, 0);
	}

	if(pHeader->command == IC_KD_IR_LOG && dataLength >= sizeof(IrLog))
	{
		IrLog* pLog = (IrLog*)pData;

		presenter.lastIrLog = *pLog;

		EventClientSend(presenter.pEventClient, EVENT_IR_UPDATE, 0);
	}
}

void PresenterInit(EventManager* pEventManager)
{
	init(0);

	presenter.pEventClient = EventClientInit(pEventManager, &eventSink, &periodic, MS2ST(20), 0);
}

void cbDesktop(WM_MESSAGE* pMsg)
{
	switch(pMsg->MsgId)
	{
		case WM_MENU:
		{
			MENU_MSG_DATA* pMenuData = (MENU_MSG_DATA*)pMsg->Data.p;
			if(pMenuData->MsgType == MENU_ON_ITEMSELECT)
			{
				switch(pMenuData->ItemId)
				{
					case ID_MENU_WIFI_BOTID:
					{
						WM_ShowWindow(presenter.hSelectBotID);
						WM_SelectWindow(presenter.hSelectBotID);
					}
					break;
					case ID_MENU_WIFI_SAVE:
					{
						IntercomHeader iHeader;
						iHeader.address = INTERCOM_ADDRESS_MAIN;
						iHeader.section = INTERCOM_SECTION_UI;
						iHeader.command = IC_UI_WIFI_SAVE;

						IntercomSend(&intercom1, &iHeader, 0, 0);
//						ConsolePrint("Saving wifi\r\n");
					}
					break;
					case ID_MENU_WIFI_SETTINGS:
					{
					}
					break;
				}
				break;
			}

			WM_DefaultProc(pMsg);
		}
		break;

		case WM_PAINT:
		{
			GUI_SetBkColor(GUI_BLACK);
			GUI_Clear();
		}
		break;

		default:
			WM_DefaultProc(pMsg);
	}
}

static void init(void* pUserData)
{
	(void)pUserData;

	GUI_Init();

	FRAMEWIN_SetDefaultTitleHeight(18);
	FRAMEWIN_SetDefaultFont(&GUI_Font16B_ASCII);

	WM_SetCallback(WM_HBKWIN, &cbDesktop);

	NumpadInit();
	NumpadSetVisible(0);

	DesktopInit();

	presenter.hSelectBotID = SelectBotIDInit();

//	EDIT_Handle hEditTest = EDIT_CreateEx(180, 120, 60, 20, 0, WM_CF_SHOW, 0, GUI_ID_EDIT0, 5);

	static MENU_ITEM_DATA wifiBotId;
	MENU_SetDefaultFont(&GUI_Font16B_ASCII);

	MENU_Handle hWifiMenu = MENU_CreateEx(0, 0, 0, 0, 0, 0, MENU_CF_VERTICAL, 0);

	wifiBotId.pText = "Bot ID";
	wifiBotId.Id = ID_MENU_WIFI_BOTID;
	MENU_AddItem(hWifiMenu, &wifiBotId);

	static MENU_ITEM_DATA wifiSettings;
	wifiSettings.pText = "Settings";
	wifiSettings.Id = ID_MENU_WIFI_SETTINGS;
	MENU_AddItem(hWifiMenu, &wifiSettings);

	static MENU_ITEM_DATA wifiSep;
	wifiSep.Flags = MENU_IF_SEPARATOR;
	wifiSep.Id = 0;
	wifiSep.pText = 0;
	MENU_AddItem(hWifiMenu, &wifiSep);

	static MENU_ITEM_DATA wifiSave;
	wifiSave.Flags = 0;
	wifiSave.pText = "Save";
	wifiSave.Id = ID_MENU_WIFI_SAVE;
	MENU_AddItem(hWifiMenu, &wifiSave);

	MENU_Handle hMainMenu = MENU_CreateEx(0, 0, 240, 20, 0, WM_CF_SHOW, MENU_CF_HORIZONTAL,  0);

	static MENU_ITEM_DATA wifiMenu;
	wifiMenu.pText = "Wifi";
	wifiMenu.Id = 0;
	wifiMenu.Flags = 0;
	wifiMenu.hSubmenu = hWifiMenu;
	MENU_AddItem(hMainMenu, &wifiMenu);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &supplyCallback;
	client.section = INTERCOM_SECTION_SUPPLY;
	IntercomAddClient(&intercom1, client);

	client.rxCallback = &kdCallback;
	client.section = INTERCOM_SECTION_KD;
	IntercomAddClient(&intercom1, client);

	client.rxCallback = &presenterCallback;
	client.section = INTERCOM_SECTION_PRESENTER;
	IntercomAddClient(&intercom1, client);
}

static void eventSink(void* pUserData, uint16_t event)
{
	(void)pUserData;

	GUI_SetFont(&GUI_Font8x12_ASCII);

	switch(event)
	{
	case EVENT_POWER_UPDATE:
	{
		DesktopPowerUpdate(&presenter.lastPowerLog);
	}
	break;
	case EVENT_KICKER_UPDATE:
	{
		DesktopKickerUpdate(&presenter.lastKickerLog);
	}
	break;
	case EVENT_IR_UPDATE:
	{
	}
	break;
	case EVENT_CONFIG_UPDATE:
	{
		DesktopWifiUpdate(&presenter.lastConfig);
	}
	break;
	default: break;
	}
}

static void periodic(void* pUserData)
{
	(void)pUserData;

	GUI_Exec();
	GUI_X_ExecIdle();
}
