#include "usb_host.h"
#include "usbh_core.h"
#include "usbh_msc.h"

#include "ff.h"
#include "ff_gen_drv.h"
#include "usbh_diskio.h" /* defines USBH_Driver as external */

#include "ch.h"

/* USB Host Core handle declaration */
USBH_HandleTypeDef hUsbHostFS;
ApplicationTypeDef Appli_state = APPLICATION_IDLE;

uint8_t USBH_DriverNum; /* FatFS USBH part */
char USBH_Path[4]; /* USBH card logical drive path */

extern HCD_HandleTypeDef hhcd_USB_OTG_FS;

void OTG_FS_IRQHandler(void)
{
	CH_IRQ_PROLOGUE();

	HAL_NVIC_ClearPendingIRQ(OTG_FS_IRQn);
	HAL_HCD_IRQHandler(&hhcd_USB_OTG_FS);

	CH_IRQ_EPILOGUE();
}

/*
 * user callbak definition
 */
static void USBH_UserProcess(USBH_HandleTypeDef *pHost, uint8_t id)
{
	(void)pHost;

	/* USER CODE BEGIN 2 */
	switch (id)
	{
	case HOST_USER_SELECT_CONFIGURATION:
		break;

	case HOST_USER_DISCONNECTION:
		Appli_state = APPLICATION_DISCONNECT;
		break;

	case HOST_USER_CLASS_ACTIVE:
		Appli_state = APPLICATION_READY;
		break;

	case HOST_USER_CONNECTION:
		Appli_state = APPLICATION_START;
		break;

	default:
		break;
	}
	/* USER CODE END 2 */
}

void USBHInit()
{
	// Link FatFS driver
	USBH_DriverNum = FATFS_LinkDriver(&USBH_Driver, USBH_Path);

	// Init Host Library, Add Supported Class and Start the library
	USBH_Init(&hUsbHostFS, USBH_UserProcess, HOST_FS);
	USBH_RegisterClass(&hUsbHostFS, USBH_MSC_CLASS);
	USBH_Start(&hUsbHostFS);
}
