/*
 * motors.c
 *
 *  Created on: 10.06.2014
 *      Author: AndreR
 */

#include "motors.h"
#include "intercom1.h"
#include "intercom_constants.h"
#include "drive.h"
#include "ctrl_foc.h"
#include "ctrl_6step.h"

void MotorsRegularIntercom()
{
	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MAIN;
	iHeader.section = INTERCOM_SECTION_MOTORS;
	iHeader.command = IC_MOTORS_MOT2MAIN;

	MotorsMot2Main mot2Main;
#ifdef MOT_RIGHT
	mot2Main.source = MOT_SOURCE_RIGHT;
#else
	mot2Main.source = MOT_SOURCE_LEFT;
#endif
	mot2Main.mot[MOT_FRONT].velocity = pFront->curVelocity;
	mot2Main.mot[MOT_REAR].velocity = pRear->curVelocity;

	IntercomSend(&intercom1, &iHeader, sizeof(mot2Main), (uint8_t*)&mot2Main);
}

static void motorsCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_MOTORS_MAIN2MOT && dataLength >= sizeof(MotorsMain2Mot))
	{
		MotorsMain2Mot* pMain2Mot = (MotorsMain2Mot*)pData;

		pFront->curBat = pMain2Mot->bat;
		pRear->curBat = pMain2Mot->bat;

		float vFront = pMain2Mot->mot[MOT_FRONT].velocity;
		float vRear = pMain2Mot->mot[MOT_REAR].velocity;

		if(vFront > DRIVE_MOVE_THRESHOLD || vFront < -DRIVE_MOVE_THRESHOLD)
			DriveSetpointInfo(pFront, 0);
		else
			DriveSetpointInfo(pFront, 1);

		if(vRear > DRIVE_MOVE_THRESHOLD || vRear < -DRIVE_MOVE_THRESHOLD)
			DriveSetpointInfo(pRear, 0);
		else
			DriveSetpointInfo(pRear, 1);

		focFront.setVelocity = vFront;
		focRear.setVelocity = vRear;

		stepFront.setVelocity = vFront;
		stepRear.setVelocity = vRear;
	}

	if(pHeader->command == IC_MOTORS_CTRL_TEMP_MEAS && dataLength >= sizeof(MotorsCtrlTempMeas))
	{
		MotorsCtrlTempMeas* pCtrl = (MotorsCtrlTempMeas*)pData;

		pFront->enableTempMeas = pCtrl->enable;
		pRear->enableTempMeas = pCtrl->enable;
	}
}

void MotorsInit()
{
	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &motorsCallback;
	client.section = INTERCOM_SECTION_MOTORS;

	IntercomAddClient(&intercom1, client);
}
