/*
 * commands.h
 *
 *  Created on: 24.05.2010
 *      Author: AndreR
 */

#ifndef __COMMANDS_H__
#define __COMMANDS_H__

#include <stdint.h>
#include "util/float16_t.h"

#include "util/hermite_spline.h"

#define PACKED __attribute__ ((packed))

#define PACKET_HEADER_SIZE 2
typedef union _PacketHeader
{
	struct
	{
		uint8_t cmd;
		uint8_t section;
	};

	uint8_t _data[PACKET_HEADER_SIZE];
} PacketHeader;

// SECTIONS
#define SECTION_SYSTEM			0
#define SECTION_DECT			1
#define SECTION_MOTOR			2
#define SECTION_KICKER			4
#define SECTION_MOVEMENT		5
#define SECTION_MULTICAST		6
#define SECTION_CTRL			7
#define SECTION_BASE_STATION	8
#define SECTION_BOOTLOADER		9
#define SECTION_SKILL			10

// SYSTEM
#define CMD_SYSTEM_STATUS_MOVEMENT 3
#define LEN_SYSTEM_STATUS_MOVEMENT 12

#define CMD_SYSTEM_POWER_LOG 2
#define LEN_SYSTEM_POWER_LOG 14

#define CMD_SYSTEM_SET_IDENTITY 4
#define LEN_SYSTEM_SET_IDENTITY 27

#define CMD_SYSTEM_ANNOUNCEMENT 5
#define LEN_SYSTEM_ANNOUNCEMENT 12

#define CMD_SYSTEM_PING 6
#define LEN_SYSTEM_PING 4

#define CMD_SYSTEM_PONG 7
#define LEN_SYSTEM_PONG 4

#define CMD_SYSTEM_BIG_PING 8
#define LEN_SYSTEM_BIG_PING 200

#define CMD_SYSTEM_SET_LOGS 9
#define LEN_SYSTEM_SET_LOGS 9

#define CMD_SYSTEM_CONSOLE_PRINT 10

#define CMD_SYSTEM_CONSOLE_COMMAND 11

#define CMD_SYSTEM_STATUS_V2 0x80
#define LEN_SYSTEM_STATUS_V2 22

#define CMD_SYSTEM_STATUS_EXT 0x81
#define LEN_SYSTEM_STATUS_EXT 40

// KICKER
#define CMD_KICKER_KICKV1 1
#define LEN_KICKER_KICKV1 4

#define CMD_KICKER_STATUS 2
#define LEN_KICKER_STATUS 12

#define CMD_KICKER_SET_AUTOLOAD 3
#define LEN_KICKER_SET_AUTOLOAD 2

#define CMD_KICKER_SET_MAX_CAP_LEVEL 4
#define LEN_KICKER_SET_MAX_CAP_LEVEL 2

#define CMD_KICKER_CHARGE 5
#define LEN_KICKER_CHARGE 2

#define CMD_KICKER_SET_DUTY_CYCLE 6
#define LEN_KICKER_SET_DUTY_CYCLE 4

#define CMD_KICKER_KICKV2 7
#define LEN_KICKER_KICKV2 4

#define CMD_KICKER_STATUSV2 8
#define LEN_KICKER_STATUSV2 12

#define CMD_KICKER_CHARGE_MANUAL 9
#define LEN_KICKER_CHARGE_MANUAL 6

#define CMD_KICKER_CHARGE_AUTO 10
#define LEN_KICKER_CHARGE_AUTO 2

#define CMD_KICKER_IR_LOG 11
#define LEN_KICKER_IR_LOG 8

#define CMD_KICKER_STATUSV3 12
#define LEN_KICKER_STATUSV3 6

// MOTOR
#define CMD_MOTOR_MOVE 0
#define LEN_MOTOR_MOVE 6

#define CMD_MOTOR_DRIBBLE 1
#define LEN_MOTOR_DRIBBLE 2

#define CMD_MOTOR_SET_PID_PARAMS 2
#define LEN_MOTOR_SET_PID_PARAMS 21

#define CMD_MOTOR_SET_MODE 3
#define LEN_MOTOR_SET_MODE 3

#define CMD_MOTOR_SET_MANUAL 4
#define LEN_MOTOR_SET_MANUAL 3

#define CMD_MOTOR_SET_PID_SP 5
#define LEN_MOTOR_SET_PID_SP 5

#define CMD_MOTOR_PID_LOG 6
#define LEN_MOTOR_PID_LOG 28

#define CMD_MOTOR_SET_PARAMS 7
#define LEN_MOTOR_SET_PARAMS 81

#define CMD_MOTOR_MOVE_V2 8
#define LEN_MOTOR_MOVE_V2 8

// MOVEMENT
#define CMD_MOVEMENT_LIS3_LOG_RAW 0
#define LEN_MOVEMENT_LIS3_LOG_RAW 12

#define CMD_MOVEMENT_LIS3_LOG 1
#define LEN_MOVEMENT_LIS3_LOG 24

#define CMD_MOVEMENT_SET_PARAMS 2
#define LEN_MOVEMENT_SET_PARAMS 48

// MULTICAST
#define CMD_MULTICAST_UPDATE_ALL 0
#define LEN_MULTICAST_UPDATE_ALL 78

#define CMD_MULTICAST_UPDATE_ALL_V2 1
#define LEN_MULTICAST_UPDATE_ALL_V2 91

// CONTROL
#define CMD_CTRL_VISION_POS 0
#define LEN_CTRL_VISION_POS 6

#define CMD_CTRL_SPLINE_2D 1
#define LEN_CTRL_SPLINE_2D 19

#define CMD_CTRL_SPLINE_1D 2
#define LEN_CTRL_SPLINE_1D 11

#define CMD_CTRL_SET_FILTER_PARAMS 3
#define LEN_CTRL_SET_FILTER_PARAMS 13

#define CMD_CTRL_SET_PID_PARAMS 4
#define LEN_CTRL_SET_PID_PARAMS 13

#define CMD_CTRL_SET_CONTROLLER_TYPE 5
#define LEN_CTRL_SET_CONTROLLER_TYPE 1

#define CMD_CTRL_VISION_TRACKED_OBJ 6

#define CMD_CTRL_RESET 7

// BASE_STATION
#define CMD_BASE_STATION_ACOMMAND 0	// this command is varying length, thus no LEN define
#define CMD_BASE_STATION_PING 1 // varying length too

#define CMD_BASE_STATION_AUTH 2

#define CMD_BASE_STATION_STATS 3

#define CMD_BASE_STATION_CONFIG 4

#define CMD_BASE_STATION_VISION_CONFIG 5

// BOOTLOADER
#define CMD_BOOTLOADER_COMMAND 0
#define LEN_BOOTLOADER_COMMAND 1

#define CMD_BOOTLOADER_DATA 1
#define LEN_BOOTLOADER_DATA 264

#define CMD_BOOTLOADER_RESPONSE 2
#define LEN_BOOTLOADER_RESPONSE 5


typedef struct PACKED _KickerStatus
{
	uint16_t chg;
	uint16_t disStr;
	uint16_t disChip;
	uint16_t cap;
	uint16_t tempStr;
	uint16_t tempChip;
} KickerStatus;

typedef struct PACKED _KickerKickV1
{
	uint8_t device;
	uint8_t mode;
	uint16_t duration;
} KickerKickV1;

typedef struct PACKED _KickerCharge
{
	uint16_t time;
} KickerCharge;

typedef struct PACKED _KickerSetDutyCycle
{
	uint16_t on;
	uint16_t off;
} KickerSetDutyCycle;

typedef struct PACKED _CMDKickerSetAutoload
{
	uint8_t log;
	uint8_t mode;
} CMDKickerSetAutoload;

typedef struct PACKED _CMDKickerSetMaxCapLevel
{
	uint16_t max;
} CMDKickerSetMaxCapLevel;

typedef struct PACKED _KickerKickV2	// 4
{
	uint8_t device:4;
	uint8_t mode:4;
	uint8_t level;
	uint16_t duration;
} KickerKickV2;

typedef struct PACKED _KickerStatusV2	// 12
{
	uint16_t chg;
	uint16_t cap;
	uint16_t TDiode[2];
	uint16_t TIGBT[2];
} KickerStatusV2;

typedef struct PACKED _KickerStatusV3	// 6
{
	uint16_t chg;
	uint16_t cap;
	uint16_t ir;
} KickerStatusV3;

typedef struct PACKED _KickerChargeManual	// 6
{
	uint16_t tOnTicks;
	uint16_t tOffTicks;
	uint16_t duration; // in 100us steps
} KickerChargeManual;

typedef struct PACKED _KickerChargeAuto	// 2
{
	uint16_t maxCapLevel;
} KickerChargeAuto;

typedef struct PACKED _KickerIrLog	// 8
{
	uint16_t voltage[4];
} KickerIrLog;

typedef struct PACKED _MotorMove	// 6
{
	int16_t vx; // [mm/s]
	int16_t vy;	// [mm/s]
	int16_t vw;	//outer rotation speed [mm/s]
} MotorMove;

typedef struct PACKED _MotorMoveV2	// 8
{
	int16_t vx;
	int16_t vy;
	int16_t vw;
	int16_t vv;
} MotorMoveV2;

typedef struct PACKED _MotorDribble	// 2
{
	int16_t rpm;
} MotorDribble;

typedef struct PACKED _MotorSetPidParams // 21
{
	uint8_t id;
	int32_t Kp;
	int32_t Ki;
	int32_t Kd;
	int32_t slewMax;
	int32_t frictionKick;
} MotorSetPidParams;

typedef struct PACKED _MotorSetParams	// 81
{
	uint8_t mode;				// 1
	int32_t Kp[5];				// 20
	int32_t Ki[5];				// 20
	int32_t Kd[5];				// 20
	int32_t slewMax[5];			// 20
} MotorSetParams;

typedef struct PACKED _SystemSetLogs	// 9
{
	uint8_t motor[5];
	uint8_t kicker;
	uint8_t power;
	uint8_t movement;
	uint8_t extMovement;
	uint8_t accel;
	uint8_t ir;
} SystemSetLogs;

typedef struct PACKED _MotorSetMode // 3
{
	uint8_t id;
	uint8_t mode;
	uint8_t log;
} MotorSetMode;

typedef struct PACKED _MotorSetManual	// 3
{
	uint8_t id;
	int16_t power;
} MotorSetManual;

typedef struct PACKED _MotorSetPidSp	// 5
{
	uint8_t id;
	int32_t setpoint;
} MotorSetPidSp;

typedef struct PACKED _MotorPidLog		// 28
{
	uint8_t id;
	int32_t latest;
	int32_t setpoint;
	int32_t pError;
	int32_t iError;
	int32_t dError;
	int32_t output;
	uint8_t overload;
	uint16_t eCurrent;
} MotorPidLog;

typedef struct PACKED _SystemStatusMovement	// 12
{
	int32_t vx;	// [mm/s]
	int32_t vy;
	int32_t vw;
} SystemStatusMovement;

typedef struct PACKED _SystemPowerLog	// 14
{
	uint16_t iVCC;
	uint16_t i5V;
	uint16_t i3V3;
	uint16_t u1;
	uint16_t u2;
	uint16_t u3;
	uint16_t u4;
} SystemPowerLog;

typedef struct PACKED _SystemSetIdentity	// 27
{
	uint32_t	cpuId[3];
	uint8_t		mac[6];
	uint8_t		ip[4];
	uint16_t	port;
	uint16_t	serverPort;
	uint8_t		botId;
} SystemSetIdentity;

typedef struct PACKED _SystemAnnouncement	// 12
{
	uint32_t	cpuId[3];
} SystemAnnouncement;

typedef struct PACKED _SystemPing	// 4
{
	uint32_t id;
} SystemPing;

typedef struct PACKED _SystemPong	// 4
{
	uint32_t id;
} SystemPong;

typedef struct PACKED _SystemBigPing	// 200
{
	uint32_t id;
	uint8_t dummy[196];
} SystemBigPing;

typedef struct PACKED _SystemConsolePrint	// 1 + n
{
	uint8_t source;		// 1
} SystemConsolePrint;

typedef struct PACKED _SystemConsoleCommand	// 1 + n
{
	uint8_t target;		// 1
} SystemConsoleCommand;

typedef struct PACKED _MulticastUpdateAll	// 78
{
	uint8_t botId[6];		// 6 * 1 = 6
	MotorMove move[6];		// 6 * 6 = 36
	KickerKickV2 kick[6];	// 6 * 4 = 24
	MotorDribble dribble[6];	// 6 * 2 = 12
} MulticastUpdateAll;

typedef struct PACKED _MulticastUpdateAllV2	// 91
{
	uint8_t botId[6];		// 6 * 1 = 6
	MotorMoveV2 move[6];	// 6 * 8 = 48
	KickerKickV2 kick[6];	// 6 * 4 = 24
	MotorDribble dribble[6];	// 6 * 2 = 12
	uint8_t useMove;		// 1 (bitfield)
} MulticastUpdateAllV2;

typedef struct PACKED _MovementLis3LogRaw	// 12
{
	int16_t iax;
	int16_t iay;
	int16_t iaz;
	int16_t oax;
	int16_t oay;
	int16_t oaz;
} MovementLis3LogRaw;

typedef struct PACKED _MovementLis3Log	// 24
{
	int32_t ax;
	int32_t ay;
	int32_t aw;
	int32_t vx;
	int32_t vy;
	int32_t vw;
} MovementLis3Log;

typedef struct PACKED _MovementSetParams	// 48
{
	int32_t Kp[3];		// 12
	int32_t Ki[3];		// 12
	int32_t Kd[3];		// 12
	int32_t slew[3];	// 12
} MovementSetParams;

typedef struct PACKED _SystemStatusV2	//	22
{
	uint16_t kickerLevel; // [Ve-1]			2
	uint8_t barrierInterrupted; // 1 or 0	1
	uint8_t bits;	// bitmask: pos, vel, acc, dribbler reached, dribbler overload	1
	int16_t position[3];	// [mm]			6
	int16_t velocity[3];	// [mm/s]		6
	int16_t acceleration[3];	// [mm/s�]	6
} SystemStatusV2;

typedef struct PACKED _SystemStatusExtended	//	64
{
	uint16_t kickerLevel; // [Ve-1]			2
	uint8_t barrierInterrupted; // 1 or 0	1
	uint8_t bits;	// bitmask: pos, vel, acc, dribbler reached, dribbler overload	1
	int16_t position[3];	// [mm]			6
	int16_t velocity[3];	// [mm/s]		6
	int16_t acceleration[3];	// [mm/s^2]	6
	int16_t targetPos[3];	// [mm]			6
	int16_t targetVel[3];	// [mm/s]		6
	int16_t targetAcc[3];	// [mm/s^2]	6
	int16_t motorSet[4];	// [mm/s^2]	8
	int16_t motorCur[4];	// [mm/s^2]	8
	int16_t motorOut[4];	// [mm/s^2]	8
} SystemStatusExtended;

typedef struct PACKED _CtrlVisionPos	// 6
{
	int16_t pos[3];	// [mm], [mrad]
} CtrlVisionPos;

typedef struct PACKED _CtrlVisionTrackedObj	// 6
{
	int16_t pos[3];	// [mm], [mrad]
} CtrlVisionTrackedObj;

#define CTRL_SPLINE_OPTION_APPEND	0x01
#define CTRL_SPLINE_OPTION_CLEAR	0x02


typedef struct PACKED _CtrlSpline2D		// 27
{
	struct _splines
	{
		float16_t a[SPLINE_SIZE]; // SPLINE_SIZE * 2
	} splines[2]; // 12*2 = 24

	float16_t tEnd; // 2

	uint8_t options; // 1
} CtrlSpline2D;

typedef struct PACKED _CtrlSpline1D		// 11
{
		float16_t a[SPLINE_SIZE]; // SPLINE_SIZE * 2

	float16_t tEnd; // 2

	uint8_t options; // 1
} CtrlSpline1D;

#define CTRL_FILTER_PARAM_EX_POS		0x01
#define CTRL_FILTER_PARAM_EX_VEL		0x02
#define CTRL_FILTER_PARAM_EX_ACC		0x03
#define CTRL_FILTER_PARAM_EZ_VISION		0x04
#define CTRL_FILTER_PARAM_EZ_ENCODER	0x05
#define CTRL_FILTER_PARAM_EZ_ACC_GYRO	0x06
#define CTRL_FILTER_PARAM_EZ_MOTOR		0x07

typedef struct PACKED _CtrlSetFilterParams // 13
{
	float params[3];	// X, Y, W
	uint8_t type;
} CtrlSetFilterParams;

#define CTRL_PID_PARAM_POS_X 0x01
#define CTRL_PID_PARAM_POS_Y 0x02
#define CTRL_PID_PARAM_POS_W 0x03
#define CTRL_PID_PARAM_VEL_X 0x04
#define CTRL_PID_PARAM_VEL_Y 0x05
#define CTRL_PID_PARAM_VEL_W 0x06
#define CTRL_PID_PARAM_MOTOR 0x07
#define CTRL_PID_PARAM_SPLINE_X 0x0B
#define CTRL_PID_PARAM_SPLINE_Y 0x0C
#define CTRL_PID_PARAM_SPLINE_W 0x0D

typedef struct PACKED _CtrlSetPIDParams // 13
{
	float Kp;
	float Ki;
	float Kd;
	uint8_t type;
} CtrlSetPIDParams;

#define CTRL_TYPE_FUSION_VEL	1
#define CTRL_TYPE_FUSION		2

typedef struct PACKED _CtrlSetControllerType // 1
{
	uint8_t controllerId;
} CtrlSetControllerType;

typedef struct PACKED _BaseStationACommand
{
	uint8_t id;
	uint8_t* pCommand;
} BaseStationACommand;

typedef struct PACKED _BaseStationPing
{
	uint32_t id;
	uint8_t* pData;
} BaseStationPing;

typedef struct PACKED _BaseStationAuth	// 4
{
	uint32_t key;	// 0x42424242
} BaseStationAuth;

#define BASE_STATION_STATS_NUM_BOTS 12	// TODO: extend to 24 and make variable command

typedef struct PACKED _BaseStationStats // 418
{
	struct PACKED	// 33*12 = 396
	{
		uint8_t botId;

		uint8_t rxLoss;			// 0 - 200
		uint8_t txLoss;			// 0 - 200
		uint16_t linkQuality;	// 0 - 40000
		uint16_t rxLevel;		// 0 - 40000
		uint16_t txLevel;		// 0 - 40000

		uint32_t txBytesSentRaw;
		uint32_t txBytesSentCOBS;
		uint32_t rxBytesRecvRaw;
		uint32_t rxBytesRecvCOBS;

		uint32_t txPacketsSent;
		uint32_t rxPacketsRecv;
	} bot[BASE_STATION_STATS_NUM_BOTS];

	struct PACKED	// 20
	{
		uint8_t rxLevel;	// 0 - 200
		uint8_t txLevel;	// 0 - 200
		uint16_t rxLoss;	// 0 - 40000

		uint32_t txBytesSent;
		uint32_t rxBytesRecv;

		uint32_t txFramesSent;
		uint32_t rxFramesRecv;

		uint16_t updateRate;	// [Hz]
	} eth;

} BaseStationStats;

typedef struct PACKED _BaseStationConfig	// 7
{
	uint8_t channel;		// frequency
	uint8_t invertPosition;
	uint8_t maxBots;
	uint16_t maxVisionRate;
	uint16_t timeout;
} BaseStationConfig;

#define BOOTLOADER_CMD_MODE_QUERY	0
#define BOOTLOADER_CMD_ENTER		1
#define BOOTLOADER_CMD_EXIT			2
#define BOOTLOADER_CMD_ERASE_MAIN	3
#define BOOTLOADER_CMD_ERASE_MEDIA	4
#define BOOTLOADER_CMD_ERASE_KD		5
#define BOOTLOADER_CMD_ERASE_LEFT	6
#define BOOTLOADER_CMD_ERASE_RIGHT	7

typedef struct PACKED _BootloaderCommand
{
	uint8_t cmd; // enter BL, exit BL, erase/prepare (select processor), mode query
} BootloaderCommand;

typedef struct PACKED _BootloaderData	// 264
{
	uint32_t offset;	// 4
	uint32_t data[64];	// 256
	uint32_t crc;		// 4
} BootloaderData;

#define BOOTLOADER_RESPONSE_ACK				0
#define BOOTLOADER_RESPONSE_NACK			1
#define BOOTLOADER_RESPONSE_MODE_NORMAL		2
#define BOOTLOADER_RESPONSE_MODE_BOOTLOADER	3

typedef struct PACKED _BootloaderResponse	// 5
{
	uint8_t type; // ACK, NACK, mode
	uint32_t offset;
} BootloaderResponse;

#define CMD_SKILL_KEEPER 0
#define CMD_SKILL_CALIBRATE_ENC 1
#define CMD_SKILL_SHOOTER 2
#define CMD_SKILL_POSITIONING 3
#define CMD_SKILL_CALIBRATE_MOTOR 4

typedef struct PACKED _SkillKeeperCommand // 6
{
	int16_t point2Block[2]; //[mm]
	int16_t radius; // [mm]
} SkillKeeperCommand;

typedef struct PACKED _SkillShooterCommand // 7
{
	int16_t shootTarget[2]; // [mm]
	uint16_t duration;
	uint8_t device;
} SkillShooterCommand;

typedef struct PACKED _SkillPositioningCommand // 6
{
	int16_t destination[3]; // x,y,w [mm]
} SkillPositioningCommand;

typedef struct PACKED _SkillCalibrateMotorCommand // 6
{
	int16_t alpha; // learn factor
	int16_t speed[3]; // velocity to use
	int16_t length; // time to drive
} SkillCalibrateMotorCommand;


#endif /* COMMANDS_H_ */
