/*
 * flash.c
 *
 *  Created on: 12.06.2014
 *      Author: AndreR
 */

#include "util/flash.h"

#include "stm32f30x_iwdg.h"
#include "stm32f30x_flash.h"

#include "ch.h"

FLASH_Status waitForLastOperation()
{
	FLASH_Status status = FLASH_COMPLETE;

	/* Check for the FLASH Status */
	status = FLASH_GetStatus();

	/* Wait for the FLASH operation to complete by polling on BUSY flag to be reset.
     Even if the FLASH operation fails, the BUSY flag will be reset and an error
     flag will be set */
	while(status == FLASH_BUSY)
	{
		status = FLASH_GetStatus();
		chThdSleep(MS2ST(1));
	}

	/* Return the operation status */
	return status;
}

FLASH_Status erasePage(uint32_t Page_Address)
{
	FLASH_Status status = FLASH_COMPLETE;

	/* Wait for last operation to be completed */
	status = waitForLastOperation();

	if (status == FLASH_COMPLETE)
	{
		/* If the previous operation is completed, proceed to erase the page */
		FLASH->CR |= FLASH_CR_PER;
		FLASH->AR = Page_Address;
		FLASH->CR |= FLASH_CR_STRT;

		/* Wait for last operation to be completed */
		status = waitForLastOperation();

		/* Disable the PER Bit */
		FLASH->CR &= ~FLASH_CR_PER;
	}

	/* Return the Erase Status */
	return status;
}

int16_t erase(uint32_t firstAddr, uint32_t lastAddr)
{
	firstAddr &= 0xFFFFF800;

	// Clear pending flags (if any)
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPERR);

	for(uint32_t addr = firstAddr; addr < lastAddr; addr += 0x800)
	{
		if(erasePage(addr) != FLASH_COMPLETE)
			return 1;
	}

	return 0;
}

void FlashErase(uint32_t addr, uint32_t endAddr)
{
	FLASH_Unlock();

	if(erase(addr, endAddr))
		return;

	FLASH_Lock();
}

void FlashProgram(uint32_t addr,  uint32_t* pData, uint32_t length)
{
	FLASH_Unlock();

	// Clear pending flags (if any)
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPERR);

	for(uint32_t i = 0; i < length; i++)
	{
		// TODO: use IRQs/task delays
		FLASH_ProgramWord(addr, pData[i]);
		addr += 4;
	}

	FLASH_Lock();
}
