/*
 * flash.c
 *
 *  Created on: 12.06.2014
 *      Author: AndreR
 */

#include "util/flash.h"

#include "stm32f4xx_iwdg.h"
#include "stm32f4xx_flash.h"

#include "ch.h"

FLASH_Status waitForLastOperation()
{
	FLASH_Status status = FLASH_COMPLETE;

	/* Check for the FLASH Status */
	status = FLASH_GetStatus();

	/* Wait for the FLASH operation to complete by polling on BUSY flag to be reset.
     Even if the FLASH operation fails, the BUSY flag will be reset and an error
     flag will be set */
	while(status == FLASH_BUSY)
	{
		status = FLASH_GetStatus();
		chThdSleep(MS2ST(1));
	}

	/* Return the operation status */
	return status;
}

#define SECTOR_MASK ((uint32_t)0xFFFFFF07)

static FLASH_Status eraseSector(uint32_t sector)
{
	uint32_t tmp_psize = 0x0;
	FLASH_Status status = FLASH_COMPLETE;

	tmp_psize = FLASH_PSIZE_WORD;
	/* Wait for last operation to be completed */
	status = waitForLastOperation();

	if(status == FLASH_COMPLETE)
	{
		/* if the previous operation is completed, proceed to erase the sector */
		FLASH->CR &= CR_PSIZE_MASK;
		FLASH->CR |= tmp_psize;
		FLASH->CR &= SECTOR_MASK;
		FLASH->CR |= FLASH_CR_SER | sector;
		FLASH->CR |= FLASH_CR_STRT;

		/* Wait for last operation to be completed */
		status = waitForLastOperation();

		/* if the erase operation is completed, disable the SER Bit */
		FLASH->CR &= (~FLASH_CR_SER);
		FLASH->CR &= SECTOR_MASK;
	}

	/* Return the Erase Status */
	return status;
}

void FlashErase(uint32_t addr, uint32_t endAddr)
{
	FLASH_Unlock();

	// Clear pending flags (if any)
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_OPERR | FLASH_FLAG_WRPERR |	FLASH_FLAG_PGAERR | FLASH_FLAG_PGPERR|FLASH_FLAG_PGSERR);

	if(addr < 0x08004000)
		eraseSector(FLASH_Sector_0);	// 16k
	if(addr < 0x08008000 && endAddr > 0x08004000)
		eraseSector(FLASH_Sector_1);	// 16k
	if(addr < 0x0800C000 && endAddr > 0x08008000)
		eraseSector(FLASH_Sector_2);	// 16k
	if(addr < 0x08010000 && endAddr > 0x0800C000)
		eraseSector(FLASH_Sector_3);	// 16k
	if(addr < 0x08020000 && endAddr > 0x08010000)
		eraseSector(FLASH_Sector_4);	// 64k
	if(addr < 0x08040000 && endAddr > 0x08020000)
		eraseSector(FLASH_Sector_5);	// 128k
	if(addr < 0x08060000 && endAddr > 0x08040000)
		eraseSector(FLASH_Sector_6);	// 128k
	if(addr < 0x08080000 && endAddr > 0x08060000)
		eraseSector(FLASH_Sector_7);	// 128k

	FLASH_Lock();
}

void FlashProgram(uint32_t addr,  uint32_t* pData, uint32_t length)
{
	FLASH_Unlock();

	// Clear pending flags (if any)
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_OPERR | FLASH_FLAG_WRPERR |	FLASH_FLAG_PGAERR | FLASH_FLAG_PGPERR|FLASH_FLAG_PGSERR);

	for(uint32_t i = 0; i < length; i++)
	{
		FLASH_ProgramWord(addr, pData[i]);
		addr += 4;
	}

	FLASH_Lock();
}
