/*
 * intercom_sections.h
 *
 *  Created on: 11.11.2012
 *      Author: AndreR
 */

#ifndef INTERCOM_SECTIONS_H_
#define INTERCOM_SECTIONS_H_

#include "commands.h"

// Sections
#define INTERCOM_SECTION_CONSOLE		0x02
#define INTERCOM_SECTION_BOOTLOADER		0x04
#define INTERCOM_SECTION_PRESENTER		0x05
#define INTERCOM_SECTION_SUPPLY			0x06
#define INTERCOM_SECTION_KD				0x07
#define INTERCOM_SECTION_COMMAND		0x08
#define INTERCOM_SECTION_NETWORK		0x09
#define INTERCOM_SECTION_MOTORS			0x0A
#define INTERCOM_SECTION_UI				0x0B

// Addresses
#define INTERCOM_ADDRESS_MAIN	0x01
#define INTERCOM_ADDRESS_MEDIA	0x02
#define INTERCOM_ADDRESS_LEFT	0x04
#define INTERCOM_ADDRESS_RIGHT	0x08
#define INTERCOM_ADDRESS_KD		0x10

// BOOTLOADER
#define BOOTLOADER_SELECT	0
#define BOOTLOADER_ERASE	1
#define BOOTLOADER_DATA		2
#define BOOTLOADER_RESPONSE	3

// CONFIG PRESENTER
#define IC_PRESENTER_CONFIG			0
#define IC_PRESENTER_KICKER_STATUS	1
#define IC_PRESENTER_DISCHARGE		2
#define IC_PRESENTER_POWER			3

// UI
#define IC_UI_WIFI_BOTID	0
#define IC_UI_WIFI_SETTINGS	1
#define IC_UI_WIFI_SAVE		2

// Messages
#define IC_SUPPLY_POWER_LOG	0

#define IC_KD_KICKER_LOG	0
#define IC_KD_IR_LOG		1

#define IC_MOTORS_MAIN2MOT 	0
#define IC_MOTORS_MOT2MAIN	1
#define IC_MOTORS_TEMP		2
#define IC_MOTORS_CTRL_TEMP_MEAS 3

typedef struct __attribute__ ((packed)) _SupplyPowerLog
{
	float bat;	// [V]
	float cur;	// [A]
} SupplyPowerLog;

typedef struct __attribute__ ((packed)) _IrLog
{
	uint16_t level[2];	// IR ADC average
} IrLog;

typedef struct __attribute__ ((packed)) _KickerLog
{
	float vCap;	// [V]
	float iChg;	// [A]
	uint16_t state;	// bitfield
} KickerLog;

typedef struct _ConfigNetwork
{
	uint8_t botId;
	uint8_t channel;		// frequency
	uint8_t speed;
	uint8_t _pad;
} __attribute__((aligned(4),packed)) ConfigNetwork;

typedef struct __attribute__ ((packed)) _UIWifiBotID
{
	uint8_t id;
} UIWifiBotID;

//################### MOTORS ####################
#define MOT_FRONT	0
#define MOT_REAR 	1
#define MOT_SOURCE_RIGHT 	0
#define MOT_SOURCE_LEFT		1

typedef struct PACKED _MotorsMain2Mot
{
	struct _motx
	{
		float velocity; 	// motor rounds per second
	} mot[2];

	float bat;	// [V]
} MotorsMain2Mot;

typedef struct PACKED _MotorsMot2Main
{
	uint8_t source;	// MOT_LEFT or MOT_RIGHT

	struct _moty
	{
		float velocity;
	} mot[2];
} MotorsMot2Main;

typedef struct PACKED _MotorsTemp
{
	uint8_t left;
	uint8_t rear;

	float t[3];
} MotorsTemp;

typedef struct PACKED _MotorsCtrlTempMeas
{
	uint8_t enable;
} MotorsCtrlTempMeas;

#endif /* INTERCOM_SECTIONS_H_ */
