/*
 * block_proto.c
 *
 *  Created on: 23.01.2014
 *      Author: AndreR
 */

#include "block_proto.h"
#include <string.h>

uint16_t BlockProtoReadData(FifoLin* pRxFifo, uint8_t* pBlockData, uint32_t blockSize)
{
	uint16_t bytesRead = 0;
	uint16_t blockLength;
	uint8_t* pStorage;
	uint8_t* pData;
	int16_t result;

	do
	{
		blockLength = *((uint16_t*)(pBlockData+bytesRead));
		if(blockLength == 0 || blockLength+bytesRead > blockSize)
			break;

		bytesRead += sizeof(uint16_t);

		pData = &pBlockData[bytesRead];
		result = FifoLinReserve(pRxFifo, blockLength, &pStorage);
		if(!result)
		{
			memcpy(pStorage, pData, blockLength);
			FifoLinCommit(pRxFifo, blockLength);
		}

		bytesRead += blockLength;
	} while(bytesRead < blockSize-sizeof(uint16_t));

	*((uint16_t*)pBlockData) = 0;

	return bytesRead;
}

uint16_t BlockProtoWriteData(FifoLin* pTxFifo, uint8_t* pBlockData, uint32_t blockSize)
{
	uint16_t bytesWritten = 0;
	uint8_t* pStorage;
	int16_t result;
	uint32_t size;

	while(bytesWritten < blockSize-sizeof(uint16_t))
	{
		result = FifoLinGet(pTxFifo, &pStorage, &size);
		if(result)
			break;

		if(size+sizeof(uint16_t) > blockSize)
		{
			// that never fits, delete packet
			FifoLinDelete(pTxFifo);
			continue;
		}

		if(size+sizeof(uint16_t) > blockSize-bytesWritten)
			break;	// that does not fit in the remaining block buffer

		*((uint16_t*)&pBlockData[bytesWritten]) = size;
		bytesWritten += sizeof(uint16_t);
		memcpy(&pBlockData[bytesWritten], pStorage, size);
		bytesWritten += size;

		FifoLinDelete(pTxFifo);
	}

	if(blockSize-bytesWritten > sizeof(uint16_t))
		*((uint16_t*)&pBlockData[bytesWritten]) = 0;	// write zero just in case this is the last packet

	return bytesWritten;
}
