/*
 * bootloader.c
 *
 *  Created on: 03.06.2013
 *      Author: AndreR
 */

#include "bootloader.h"

#include "intercom_constants.h"
#include "commands.h"
//#include "util/console.h"
#include "util/boot.h"

static void sendNACK(Intercom* pIntercom, uint32_t offset)
{
	BootloaderResponse response;
	response.type = BOOTLOADER_RESPONSE_NACK;
	response.offset = offset;

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MAIN;
	header.section = INTERCOM_SECTION_BOOTLOADER;
	header.command = BOOTLOADER_RESPONSE;

	IntercomSend(pIntercom, &header, sizeof(BootloaderResponse), (uint8_t*)&response);
}

static void sendACK(Intercom* pIntercom, uint32_t offset)
{
	BootloaderResponse response;
	response.type = BOOTLOADER_RESPONSE_ACK;
	response.offset = offset;

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MAIN;
	header.section = INTERCOM_SECTION_BOOTLOADER;
	header.command = BOOTLOADER_RESPONSE;

	IntercomSend(pIntercom, &header, sizeof(BootloaderResponse), (uint8_t*)&response);
}

static void bootloaderCb(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)dataLength;

	Intercom* pIntercom = (Intercom*)pUserData;

	switch(pHeader->command)
	{
		case BOOTLOADER_ERASE:
		{
//			ConsolePrint("Erasing...");

			if(BootEraseProgramFlash())
			{
				sendNACK(pIntercom, 0);
//				ConsolePrint("failed\r\n");
			}
			else
			{
				sendACK(pIntercom, 0);
//				ConsolePrint("done\r\n");
			}
		}
		break;
		case BOOTLOADER_DATA:
		{
			BootloaderData* pBootData = (BootloaderData*)pData;

			BootProgramFlash(pBootData->data, 64, pBootData->offset);

			chThdSleep(MS2ST(20));

			sendACK(pIntercom, pBootData->offset);
		}
		break;
		case BOOTLOADER_SELECT:
		{
			BootSetBootloaderSelected(1);

//			ConsolePrint("Bootloader selected\r\n");
		}
		break;
	}
}

void BootloaderSlaveInit(Intercom* pIntercom)
{
	IntercomClient client;
	client.pUserData = pIntercom;
	client.section = INTERCOM_SECTION_BOOTLOADER;
	client.rxCallback = &bootloaderCb;

	IntercomAddClient(pIntercom, client);
}

void BootloaderSelect(Intercom* pIntercom)
{
	IntercomHeader iHeader;
	iHeader.section = INTERCOM_SECTION_BOOTLOADER;
	iHeader.address = INTERCOM_ADDRESS_MEDIA | INTERCOM_ADDRESS_KD | INTERCOM_ADDRESS_LEFT | INTERCOM_ADDRESS_RIGHT;
	iHeader.command = BOOTLOADER_SELECT;

	IntercomSend(pIntercom, &iHeader, 0, 0);
}
