/*
 * event.c
 *
 *  Created on: 06.03.2014
 *      Author: AndreR
 */

#include "event.h"

#define EVENT_PERIODIC	0xFFFF

static void timerFunc(void* param)
{
	EventClient* pClient = (EventClient*)param;

	chSysLockFromIsr();
	chMBPostI(pClient->pEventQueue, EVENT_PERIODIC | (pClient->clientId << 16));
	chVTSetI(&pClient->periodicTimer, pClient->periodicDelay, &timerFunc, pClient);
	chSysUnlockFromIsr();
}

void EventManagerInit(EventManager* pManager, int32_t* pMsgBuf, uint16_t bufLen)
{
	pManager->numClients = 0;
	chMBInit(&pManager->eventQueue, pMsgBuf, bufLen);
}

EventClient* EventClientInit(EventManager* pManager, EventFunc eventFunc, PeriodicFunc periodicFunc, systime_t periodicDelay, void* pUserData)
{
	if(pManager->numClients >= EVENT_MAX_CLIENTS)
		return 0;

	EventClient* pClient = &pManager->clients[pManager->numClients];

	pClient->clientId = pManager->numClients;
	++pManager->numClients;
	pClient->pEventQueue = &pManager->eventQueue;
	pClient->eventFunc = eventFunc;
	pClient->periodicFunc = periodicFunc;
	pClient->periodicDelay = periodicDelay;
	pClient->pUserData = pUserData;
	if(periodicFunc)
	{
		chVTSet(&pClient->periodicTimer, periodicDelay, &timerFunc, pClient);
	}

	return pClient;
}

msg_t EventClientSend(EventClient* pClient, uint32_t event, systime_t timeout)
{
	return chMBPost(pClient->pEventQueue, event | (pClient->clientId << 16), timeout);
}

msg_t EventClientSendI(EventClient* pClient, uint32_t event)
{
	return chMBPostI(pClient->pEventQueue, event | (pClient->clientId << 16));
}

msg_t EventManagerTask(void* params)
{
	EventManager* pManager = (EventManager*)params;
	msg_t msg;
	uint16_t clientId;
	uint16_t event;
	EventClient* pClient;

	chRegSetThreadName("EventManager");

	while(1)
	{
		chMBFetch(&pManager->eventQueue, &msg, TIME_INFINITE);
		clientId = (msg >> 16);
		event = (msg & 0xFFFF);
		pClient = &pManager->clients[clientId];

		if(event == EVENT_PERIODIC)
		{
			(*pClient->periodicFunc)(pClient->pUserData);
		}
		else
		{
			(*pClient->eventFunc)(pClient->pUserData, event);
		}
	}

	return 0;
}
