/*
 * fifo.c
 *
 *  Created on: 03.10.2013
 *      Author: AndreR
 */

#include "fifo.h"
#include "errors.h"
#include <string.h>

void FifoInit(Fifo* pFifo, uint8_t* pData, uint32_t size)
{
	pFifo->pFirst = pData;
	pFifo->pLast = pData+size;
	pFifo->pWrite = pData;
	pFifo->pRead = pData;
	pFifo->bytesFree = size;
	pFifo->size = size;
}

void FifoClear(Fifo* pFifo)
{
	pFifo->pWrite = pFifo->pFirst;
	pFifo->pRead = pFifo->pFirst;
	pFifo->bytesFree = pFifo->size;
}

int16_t FifoPush(Fifo* pFifo, uint8_t* pData, uint32_t size)
{
	if(pFifo->bytesFree < size)
		return ERROR_FIFO_FULL;

	uint32_t bytesToEnd = pFifo->pLast - pFifo->pWrite;

	pFifo->bytesFree -= size;

	if(bytesToEnd >= size)
	{
		memcpy(pFifo->pWrite, pData, size);
		pFifo->pWrite += size;

		if(pFifo->pWrite == pFifo->pLast)
			pFifo->pWrite = pFifo->pFirst;
	}
	else
	{
		memcpy(pFifo->pWrite, pData, bytesToEnd);
		pData += bytesToEnd;
		size -= bytesToEnd;
		memcpy(pFifo->pFirst, pData, size);
		pFifo->pWrite = pFifo->pFirst+size;
	}

	return 0;
}

int16_t FifoPop(Fifo* pFifo, uint8_t* pDst, uint32_t size)
{
	if(pFifo->size-pFifo->bytesFree < size)
		return ERROR_INVALID_PARAMETER;

	uint32_t bytesToEnd = pFifo->pLast - pFifo->pRead;

	pFifo->bytesFree += size;

	if(bytesToEnd >= size)
	{
		memcpy(pDst, pFifo->pRead, size);
		pFifo->pRead += size;

		if(pFifo->pRead == pFifo->pLast)
			pFifo->pRead = pFifo->pFirst;
	}
	else
	{
		memcpy(pDst, pFifo->pRead, bytesToEnd);
		pDst += bytesToEnd;
		size -= bytesToEnd;
		memcpy(pDst, pFifo->pFirst, size);
		pFifo->pRead = pFifo->pFirst+size;
	}

	return 0;
}

int16_t FifoDrop(Fifo* pFifo, uint32_t size)
{
	if(pFifo->size-pFifo->bytesFree < size)
		return ERROR_INVALID_PARAMETER;

	uint32_t bytesToEnd = pFifo->pLast - pFifo->pRead;

	pFifo->bytesFree += size;

	if(bytesToEnd >= size)
	{
		pFifo->pRead += size;

		if(pFifo->pRead == pFifo->pLast)
			pFifo->pRead = pFifo->pFirst;
	}
	else
	{
		pFifo->pRead = pFifo->pFirst+size-bytesToEnd;
	}

	return 0;
}

int16_t FifoFindChar(Fifo* pFifo, uint8_t ch, uint32_t* pBytesToChar)
{
	uint8_t* pRead = pFifo->pRead;
	uint32_t bytes = 0;
	*pBytesToChar = 0;

	while(pRead != pFifo->pWrite)
	{
		if(*pRead == ch)
		{
			*pBytesToChar = bytes;
			return 0;
		}

		pRead++;
		if(pRead == pFifo->pLast)
			pRead = pFifo->pFirst;

		bytes++;
	}

	return ERROR_FIFO_CHAR_NOT_FOUND;
}

uint32_t FifoGetBytesAvailable(Fifo* pFifo)
{
	return pFifo->size-pFifo->bytesFree;
}
