/*
 * fifo_delim.c
 *
 *  Created on: 03.10.2013
 *      Author: AndreR
 */

#include "fifo_delim.h"

void FifoDelimInit(FifoDelim* pFifo, uint8_t* pData, uint32_t size, uint8_t delimiter)
{
	FifoInit(&pFifo->fifo, pData, size);

	pFifo->numDelimiters = 0;
	pFifo->delimiter = delimiter;
}

void FifoDelimClear(FifoDelim* pFifo)
{
	FifoClear(&pFifo->fifo);

	pFifo->numDelimiters = 0;
}

int16_t FifoDelimPush(FifoDelim* pFifo, uint8_t* pData, uint32_t size)
{
	int16_t result = FifoPush(&pFifo->fifo, pData, size);
	if(result)
		return result;

	for(uint32_t i = 0; i < size; i++)
	{
		if(pData[i] == pFifo->delimiter)
			pFifo->numDelimiters++;
	}

	return result;
}

int16_t FifoDelimPop(FifoDelim* pFifo, uint8_t* pDst, uint32_t size)
{
	int16_t result = FifoPop(&pFifo->fifo, pDst, size);
	if(result)
		return result;

	for(uint32_t i = 0; i < size; i++)
	{
		if(pDst[i] == pFifo->delimiter)
			pFifo->numDelimiters--;
	}

	return result;
}

int16_t FifoDelimDropToDelim(FifoDelim* pFifo)
{
	uint32_t bytesToDelim;

	int16_t result = FifoDelimFindNextDelimiter(pFifo, &bytesToDelim);
	if(result)
		return result;

	result = FifoDrop(&pFifo->fifo, bytesToDelim+1);

	pFifo->numDelimiters--;

	return 0;
}

int16_t FifoDelimDrop(FifoDelim* pFifo, uint32_t size, uint32_t numDroppedDelims)
{
	int16_t result = FifoDrop(&pFifo->fifo, size);
	if(result)
		return result;

	pFifo->numDelimiters -= numDroppedDelims;

	return 0;
}

int16_t FifoDelimFindNextDelimiter(FifoDelim* pFifo, uint32_t* pSizeToDelim)
{
	return FifoFindChar(&pFifo->fifo, pFifo->delimiter, pSizeToDelim);
}
