/*
 * fifo_packet_synced.h
 *
 *  Created on: 13.02.2011
 *      Author: AndreR
 */

#ifndef __FIFO_PACKET_SYNCED_H__
#define __FIFO_PACKET_SYNCED_H__

#include "errors.h"
#include "fifo.h"
#include "fifo8_synced.h"

#include <stdint.h>

#define FIFO_PACKET_SYNCED_HEADER_SIZE 2

typedef uint8_t(*FifoPacketSyncedLockCallback)(void*);
typedef void(*FifoPacketSyncedReleaseCallback)(uint8_t, void*);

typedef struct _FifoPacketSynced
{
	Fifo fifo;

	volatile uint8_t* pLengthPosition;
	volatile uint16_t currentLength;

	FifoPacketSyncedLockCallback lockCallback;
	FifoPacketSyncedReleaseCallback releaseCallback;

	void* syncData;

	volatile uint16_t numPackets;
} FifoPacketSynced;

void		FifoPacketSyncedInit(FifoPacketSynced* pFifo, uint8_t* pBuf, uint16_t size, FifoPacketSyncedLockCallback lockCb, FifoPacketSyncedReleaseCallback releaseCb, void* pSyncData);
void		FifoPacketSyncedClear(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedBegin(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedPushByte(FifoPacketSynced* pFifo, uint8_t byte);
int16_t		FifoPacketSyncedPush(FifoPacketSynced* pFifo, uint16_t packetLength, uint8_t* pData);
int16_t		FifoPacketSyncedComplete(FifoPacketSynced* pFifo);
void		FifoPacketSyncedCancel(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedPop(FifoPacketSynced* pFifo, uint16_t* pPacketLength, uint8_t* pData, uint16_t suppliedLength);
int16_t		FifoPacketSyncedDrop(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedGetNextPacketDataLength(FifoPacketSynced* pFifo, uint16_t* pLength);
uint16_t	FifoPacketSyncedNumPackets(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedRevert(FifoPacketSynced* pFifo);
int16_t		FifoPacketSyncedCopyPacket(FifoPacketSynced* pSrcFifo, FifoPacketSynced* pDstFifo);

#endif
