/*
 * flash_fs.h
 *
 *  Created on: 25.06.2014
 *      Author: AndreR
 */

#ifndef FLASH_FS_H_
#define FLASH_FS_H_

#include <stdint.h>
#include "ch.h"

#define FLASH_FILENAME_LENGTH 16
#define FLASH_TEMP_SIZE 1024

#define FLASH_FS_OPEN 0
#define FLASH_FS_CREATED 1
#define FLASH_FS_NOMEM 2

typedef struct _FlashFile
{
	uint16_t dataSize;
	uint16_t version;
	uint32_t del;
	char filename[FLASH_FILENAME_LENGTH];
} __attribute__((aligned(4),packed)) FlashFile;

typedef struct _FlashFS
{
	uint32_t flashAddr;
	uint32_t flashSize;
	uint32_t bytesDel;
	uint32_t bytesUsed;

	uint8_t* pWrite;

	uint8_t temp[FLASH_TEMP_SIZE];

	Mutex writeMutex;
} FlashFS;

extern FlashFS flashFS;

void FlashFSInit(uint32_t flashAddr, uint32_t flashSize);
FlashFile* FlashFSOpen(const char* pFilename, uint16_t version);
FlashFile* FlashFSCreate(const char* pFilename, uint16_t version, uint16_t size);
int16_t FlashFSOpenOrCreate(const char* pFilename, uint16_t version, void* pData, uint16_t size, FlashFile** ppFH);
void FlashFSRead(FlashFile* pFH, void* pDst);
void FlashFSWrite(FlashFile** ppFH, void* pData, uint16_t size);
void FlashFSDelete(const char* pFilename);
int16_t FlashFSCompact();
void FlashFSListFiles();

#endif /* FLASH_FS_H_ */
