/*
 * intercom.c
 *
 *  Created on: 10.11.2012
 *      Author: AndreR
 */

#include "intercom.h"
#include "errors.h"
#include <string.h>

static void eventSink(void* pUserData, uint16_t event);

void IntercomRxReady(void* pUserData)
{
	Intercom* pIntercom = (Intercom*)pUserData;

	EventClientSend(pIntercom->pEventClient, 0, TIME_IMMEDIATE);
}

void IntercomInit(Intercom* pIntercom, uint8_t ownAddr, uint8_t hub, EventManager* pEventManager)
{
	pIntercom->numClientsUsed = 0;
	pIntercom->numMediaUsed = 0;
	pIntercom->ownAddress = ownAddr;
	pIntercom->hub = hub;

	pIntercom->pEventClient = EventClientInit(pEventManager, &eventSink, 0, 0, pIntercom);
}

int16_t IntercomAddMedium(Intercom* pIntercom, IntercomMedium medium)
{
	if(pIntercom->numMediaUsed == INTERCOM_MAX_MEDIA)
		return ERROR_INTERCOM_NO_FREE_SLOT;

	pIntercom->media[pIntercom->numMediaUsed] = medium;
	pIntercom->numMediaUsed++;

	return 0;
}

int16_t IntercomSend(Intercom* pIntercom, IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData)
{
	uint8_t* pDst;
	int16_t result;

	const uint32_t length = INTERCOM_HEADER_SIZE+dataLength;

	if(pHeader->address & pIntercom->ownAddress)
	{
		// look for client and forward
		for(uint8_t i = 0; i < pIntercom->numClientsUsed; i++)
		{
			if(pIntercom->clients[i].section == pHeader->section)
				(*pIntercom->clients[i].rxCallback)(pHeader, length-INTERCOM_HEADER_SIZE, pData, pIntercom->clients[i].pUserData);
		}
	}

	for(uint8_t m = 0; m < pIntercom->numMediaUsed; m++)
	{
		IntercomMedium* pMedium = &pIntercom->media[m];

		if(pMedium->reachableAddresses & pHeader->address)
		{
			result = FifoLinReserve(pMedium->pTxFifo, length, &pDst);
			if(result)
				return result;

			memcpy(pDst, pHeader->_data, INTERCOM_HEADER_SIZE);
			memcpy(pDst+INTERCOM_HEADER_SIZE, pData, dataLength);

			result = FifoLinCommit(pMedium->pTxFifo, length);
			if(result)
				return result;

			if(pMedium->txReady)
				(*pMedium->txReady)(pMedium->pTxReadyData);

		}
	}

	return 0;
}

int16_t IntercomAddClient(Intercom* pIntercom, IntercomClient client)
{
	if(pIntercom->numClientsUsed == INTERCOM_MAX_CLIENTS)
		return ERROR_INTERCOM_NO_FREE_SLOT;

	pIntercom->clients[pIntercom->numClientsUsed] = client;
	pIntercom->numClientsUsed++;

	return 0;
}

static void eventSink(void* pUserData, uint16_t event)
{
	(void)event;
	Intercom* pIntercom = (Intercom*)pUserData;

	uint32_t length;
	uint8_t* pFifoData;
	int16_t result;

	for(uint8_t m = 0; m < pIntercom->numMediaUsed; m++)
	{
		IntercomMedium* pMedium = &pIntercom->media[m];

		while(1)
		{
			result = FifoLinGet(pMedium->pRxFifo, &pFifoData, &length);
			if(result)
				break;

			IntercomHeader* pHeader = (IntercomHeader*)pFifoData;
			uint8_t* pData = pFifoData + INTERCOM_HEADER_SIZE;

			if((pHeader->address & pIntercom->ownAddress))
			{
				for(uint8_t i = 0; i < pIntercom->numClientsUsed; i++)
				{
					if(pIntercom->clients[i].section == pHeader->section)
						(*pIntercom->clients[i].rxCallback)(pHeader, length-INTERCOM_HEADER_SIZE, pData, pIntercom->clients[i].pUserData);
				}
			}

			if((pHeader->address & ~pIntercom->ownAddress) && pIntercom->hub)	// any remote recipients?
			{
				IntercomSend(pIntercom, pHeader, length-INTERCOM_HEADER_SIZE, pData);
			}

			FifoLinDelete(pMedium->pRxFifo);
		}
	}
}
