/*
 * intercom.h
 *
 *  Created on: 10.11.2012
 *      Author: AndreR
 */

#ifndef INTERCOM_H_
#define INTERCOM_H_

#include <stdint.h>

#include "util/fifo_lin.h"
#include "util/event.h"

#define INTERCOM_MAX_CLIENTS	10
#define INTERCOM_MAX_MEDIA		4

#define INTERCOM_HEADER_SIZE 3

typedef union __attribute__ ((packed)) _IntercomHeader
{
	struct
	{
		uint8_t address;
		uint8_t section;
		uint8_t command;
	};

	uint8_t _data[INTERCOM_HEADER_SIZE];
} IntercomHeader;

typedef void(*IntercomCallback)(IntercomHeader*, uint16_t, uint8_t*, void*);	//header, dataLength, data, userData

typedef struct _IntercomClient
{
	uint8_t section;
	IntercomCallback rxCallback;
	void* pUserData;
} IntercomClient;

typedef void(*IntercomTxReady)(void*);

typedef struct _IntercomMedium
{
	FifoLin* pTxFifo;
	FifoLin* pRxFifo;

	IntercomTxReady txReady;
	void* pTxReadyData;

	uint8_t reachableAddresses;
} IntercomMedium;

typedef struct _Intercom
{
	IntercomMedium media[INTERCOM_MAX_MEDIA];
	uint8_t numMediaUsed;

	IntercomClient clients[INTERCOM_MAX_CLIENTS];
	uint8_t numClientsUsed;

	EventClient* pEventClient;

	uint8_t ownAddress;
	uint8_t hub;
} Intercom;

void 	IntercomInit(Intercom* pIntercom, uint8_t ownAddr, uint8_t hub, EventManager* pEventManager);
int16_t IntercomAddMedium(Intercom* pIntercom, IntercomMedium medium);
void 	IntercomRxReady(void* pUserData);
int16_t IntercomSend(Intercom* pIntercom, IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData);
int16_t IntercomAddClient(Intercom* pIntercom, IntercomClient client);
int16_t IntercomProcess(Intercom* pIntercom, int32_t blockTime);

#endif /* INTERCOM_H_ */
